import numpy as np
from copy import deepcopy
from AvenirCommon.Util import GBRange
from SpectrumCommon.Const.VW.VWConst import *
from SpectrumCommon.Const.VW.VWTags import *
from SpectrumCommon.Const.CS import *
from SpectrumCommon.Const.CS.CSInterventionIDs import *
from SpectrumCommon.Util.CS.CSUtil import *

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def CreateArr(val, len):
    return [ deepcopy(val) for element in range( len ) ] 

def Create1DSrcArr(len):
    result = ''
    if len > 1:
        result = CreateArr('', len)
    return result

def Create2DSrcArr(SrcLengthD1, SrcLengthD2):
    return CreateArr(Create1DSrcArr(SrcLengthD2), SrcLengthD1)

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def Simple_Value_Init(value):
    return {'value' : value}

def Value_1DSource_Init(value = 0, SrcLength = 1):
    return {'value' : value, 'source' : Create1DSrcArr(SrcLength)}

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                          DBC Methods                                                     /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def Get_DBC_YearRec(DataByCountry_DB):
    return {
        'exploreLiST'      : DataByCountry_DB['exploreLiST'], 
        'DBC_FirstYr'      : DataByCountry_DB['firstYear'], 
        'DBC_FinalYr'      : DataByCountry_DB['finalYear'],
        'ProjFirstYear'    : DataByCountry_DB['ProjFirstYear'], 
        'ProjFinalYearIdx' : DataByCountry_DB['ProjFinalYearIdx']
    }

def getVal(sourceData, index):
    result = 0
    if sourceData != CS_TG_Zeros: 
        result = sourceData[index]
    return result

def getVal2(sourceData, key, index):
    result = 0
    if key in sourceData:
        result = getVal(sourceData[key], index)
    return result

def Fill_Data_DBC(DBC_YearRec, destData, sourceData, tag = ''):
    
    def getLastIndex(sourceData):
        result = -1
        if sourceData != CS_TG_Zeros: 
            result = len(sourceData)-1
        return result
    
    exploreLiST = DBC_YearRec['exploreLiST']
    DBC_FirstYr = DBC_YearRec['DBC_FirstYr']
    DBC_FinalYr = DBC_YearRec['DBC_FinalYr']
    ProjFirstYear = DBC_YearRec['ProjFirstYear']
    ProjFinalYearIdx = DBC_YearRec['ProjFinalYearIdx']

    is_GNI_Per_Cap = tag == TG_GNIPerCap
    is_Food_Insecure = tag == TG_AFBalancedEnergySupp
    
    # Set the values for the DBC years 
    index = 0
    for year in GBRange(DBC_FirstYr, DBC_FinalYr):
        t = year - ProjFirstYear
        value = getVal(sourceData, index)
        if (not is_Food_Insecure) or (value > 0):
            if (t >= 0) and (t <= ProjFinalYearIdx):
                if (not is_GNI_Per_Cap) or (year < 2020):
                    destData[t] = value
        index += 1

    # Flatline earlier years 
    value = getVal(sourceData, 0)
    if (not is_Food_Insecure) or (value > 0):
        DBCFirstYrIdx = DBC_FirstYr - ProjFirstYear
        for t in GBRange(0, DBCFirstYrIdx):
            if (t <= ProjFinalYearIdx):
                if (not is_GNI_Per_Cap) or ((ProjFirstYear + t) < 2020):
                    destData[t] = value

    # Flatline later years 
    if not is_GNI_Per_Cap:
        value = getVal(sourceData, getLastIndex(sourceData))
        if (not is_Food_Insecure) or (value > 0):
            DBCFinalYrIdx = DBC_FinalYr - ProjFirstYear
            if DBCFinalYrIdx < 0:
                DBCFinalYrIdx = 0
            for t in GBRange(DBCFinalYrIdx, ProjFinalYearIdx):
                destData[t] = value

def GetDBCBaseYearIndex(DataByCountry_DB):
    #  Find the year to read from 
    yearToRead = DataByCountry_DB['ProjFirstYear']
    if yearToRead < DataByCountry_DB['firstYear']:
        yearToRead = DataByCountry_DB['firstYear']
    if yearToRead > DataByCountry_DB['finalYear'] :
        yearToRead = DataByCountry_DB['finalYear']

    # Find the index to read from
    index = 0
    found = False
    for DBC_Year in GBRange(DataByCountry_DB['firstYear'], DataByCountry_DB['finalYear']):
        if not found and (yearToRead != DBC_Year):
            index += 1
        else:
            found = True

    return index

def Fill_Data_DBC2(destData, destKey, sourceData, index):
    destData[destKey] = getVal(sourceData, index)
    
def Fill_Data_DBC3(destData, sourceData, sourceIndex, destIndex):
    destData[destIndex] = getVal(sourceData, sourceIndex)
     
def Fill_Data_DBC4(destData, sourceData, sourceKey1, sourceKey2, destIndex, divBy100 = False):
    if not sourceData == None:
        if sourceKey1 in sourceData:
            if sourceKey2 in sourceData[sourceKey1]:
                destData[destIndex] = sourceData[sourceKey1][sourceKey2]
                if divBy100:
                    destData[destIndex]  /= 100    
    
def Fill_Source_DBC(destData, sourceData, key):
    if 'sources' in sourceData:
        if key in sourceData['sources']:
            destData['source'] = sourceData['sources'][key]

def Fill_Sources_DBC(destData, sourceData, key, index):
    if 'sources' in sourceData:
        if key in sourceData['sources']:
            destData['source'][index] = sourceData['sources'][key]

def getDBCAgeKey(a):
    if a == CS_0t1months   : return '0 to 1'
    if a == CS_1t6months   : return '1 to 6'
    if a == CS_6t12months  : return '6 to 12'
    if a == CS_12t24months : return '12 to 24'
    if a == CS_24t60months : return '24 to 60'
    return ''

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                      Default data methods                                                /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def Fill_Data(destData, destIndex, sourceData, key):
    if key in sourceData:
        destData[destIndex] = sourceData[key]

def Fill_Source(destData, sourceData, key):
    if key in sourceData:
        destData['source'] = sourceData[key]

def Fill_1D_Sources(destData, index, sourceData, key):
    if key in sourceData:
        destData['source'][index] = sourceData[key]

def Fill_2D_Sources(destData, index1, index2, sourceData, key):
    if key in sourceData:
        destData['source'][index1][index2] = sourceData[key]

def getDefaultDataAgeKey(a):
    if a == CS_0t1months   : return '0-1 months'
    if a == CS_1t6months   : return '1-6 months'
    if a == CS_6t12months  : return '6-12 months'
    if a == CS_12t24months : return '12-23 months'
    if a == CS_24t60months : return '24-59 months'
    return ''

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def CalculateQuality(IVInfo, Readiness_DB, CSection_Cov_DB, YearRec, Coverage):
    BLOOD_TAG     = '<Blood>'
    BP_TAG        = '<BP>'
    URINE_TAG     = '<Urine>'
    CSECT_TAG     = '<Csect>'
    DB_FIRSTYEAR  = 1970
    DB_FINALYEAR  = 2030
    NUM_DB_YEARS  = DB_FINALYEAR - DB_FIRSTYEAR + 1
    ANC_INDEX     = 0
    DELIV_INDEX   = 1

    #   ========================================================================
    #                 Grab the data needed for Quality calculations
    #   ========================================================================

    Data = [{}, {}]

    AnchorYear = -1
    
    if not Readiness_DB == None:   

        AnchorYear = Readiness_DB['anchorYear']

        for key, record in Readiness_DB.items():
            if type(record) == dict and 'mstID' in record and str(record['mstID']) in IVInfo:            
                mstID = str(record['mstID'])
                currID = IVInfo[mstID]['currID']
                groupMstID = IVInfo[mstID]['groupMstID']
                
                index = -1
                if   groupMstID == CS_GroupMstID_Pregnancy   : index = ANC_INDEX
                elif groupMstID == CS_GroupMstID_Childbirth  : index = DELIV_INDEX
                
                if index > -1:
                    Data[index][key] = {
                        'currID'  : currID,
                        'mstID'   : mstID,
                        'Anchor'  : record['anchor'],
                        'Gap'     : np.zeros(NUM_DB_YEARS).tolist(),
                        'Quality' : np.zeros(NUM_DB_YEARS).tolist(),
                        'Ratio'   : np.zeros(NUM_DB_YEARS).tolist()
                    }

    temp = {BLOOD_TAG : None, BP_TAG : None, URINE_TAG : None, CSECT_TAG : None}

    if not CSection_Cov_DB == None:    
        for key, record in CSection_Cov_DB.items():
            for key2, value in record.items():            
                if temp[key2] == None:
                    temp[key2] = {
                        'values'  : np.zeros(NUM_DB_YEARS).tolist(),
                        'Gap'     : np.zeros(NUM_DB_YEARS).tolist(),
                        'Quality' : np.zeros(NUM_DB_YEARS).tolist(),
                        'Ratio'   : np.zeros(NUM_DB_YEARS).tolist()
                    }

                t = 0
                for year in GBRange(DB_FIRSTYEAR, DB_FINALYEAR):
                    if year == int(key):
                        temp[key2]['values'][t] = value
                    t += 1    
    
    ancblood = temp[BLOOD_TAG]
    ancbp    = temp[BP_TAG]
    ancurine = temp[URINE_TAG]
    csectcov = temp[CSECT_TAG]

    #   ======================================================================
    #                             Calculate Quality
    #   ======================================================================

    #   We have to use all years in the database because the anchoring year
    #   may fall outside of the projection years the anchoring year is the
    #   for which their is data for the quality elements of ANC and Delivery
    #   services identification of year that will serve as the anchor for the
    #   quality associated data items year by year calculation of average
    #   tracer values AnchorYear is the year for which there is default
    #   quality data ancblood, etc. are the tracers whose evolution will set
    #   the pace at which quality improves going forward in time and declines
    #   going back ward in time. We use an average for their evolution

    if ancblood != None and ancbp != None and ancurine != None and csectcov != None:
        
        AnchorYearIdx = -1
        
        AnchorTracer = [-1] * 2
        FirstYearIdx = [-1] * 2
        FinalYearIdx = [-1] * 2
        
        Tracer       = [[0] * NUM_DB_YEARS] * 2
        GapTracer    = [[0] * NUM_DB_YEARS] * 2
        DeltaTracer  = [[0] * NUM_DB_YEARS] * 2

        t = 0
        for year in GBRange(DB_FIRSTYEAR, DB_FINALYEAR):
            if year == AnchorYear:
                AnchorYearIdx = t

            # ancblood, ancbp and ancurine are in the Readiness Trend database.
            # There can be many years of data for these items.
            if ((ancblood['values'][t] > 0) and (ancbp['values'][t] > 0) and (ancurine['values'][t] > 0)):
                value = (ancblood['values'][t] + ancbp['values'][t] + ancurine['values'][t]) / 3
                Tracer[ANC_INDEX][t] = value
                if FirstYearIdx[ANC_INDEX] < 0: 
                    FirstYearIdx[ANC_INDEX] = t
                
                FinalYearIdx[ANC_INDEX] = t

            # CSection is in the Readiness Trend database.
            if ((csectcov['values'][t] > 0) and (Tracer[ANC_INDEX][t] > 0)):
                #bill was here 2022 02
                value = (ancblood['values'][t] + ancbp['values'][t] + ancurine['values'][t] + csectcov['values'][t] * 100) / 4
                Tracer[DELIV_INDEX][t] = value
                if FirstYearIdx[DELIV_INDEX] < 0: 
                    FirstYearIdx[DELIV_INDEX] = t
                
                FinalYearIdx[DELIV_INDEX] = t
            
            t += 1       
    
        for i in GBRange(ANC_INDEX, DELIV_INDEX):
        
        #   ========================================================================
        #                          Interpolate Tracer Data
        #   ========================================================================
        
            if (FirstYearIdx[i] > -1) and (FinalYearIdx[i] > -1):
                BeginInterpolate = FirstYearIdx[i]
                EndInterpolate = FirstYearIdx[i] + 1
                while (BeginInterpolate != FinalYearIdx[i]):
                    if (Tracer[i][EndInterpolate] > 0) and (Tracer[i][EndInterpolate - 1] == 0):
                        for t in GBRange(BeginInterpolate + 1, EndInterpolate - 1):
                            Tracer[i][t] = Tracer[i][BeginInterpolate] + (Tracer[i][EndInterpolate] - Tracer[i][BeginInterpolate]) * (t - BeginInterpolate) / (EndInterpolate - BeginInterpolate)
                        BeginInterpolate = EndInterpolate
                    
                    if (Tracer[i][EndInterpolate] > 0) and (Tracer[i][EndInterpolate - 1] > 0):
                        BeginInterpolate = EndInterpolate
                    EndInterpolate = EndInterpolate + 1
            
        #   Flatlining forward and backward
            t = 0
            for year in GBRange(DB_FIRSTYEAR, DB_FINALYEAR):
                if (FirstYearIdx[i] > -1) and (FinalYearIdx[i] > -1):
                    if t < FirstYearIdx[i]: Tracer[i][t] = Tracer[i][FirstYearIdx[i]]
                    if t > FinalYearIdx[i]: Tracer[i][t] = Tracer[i][FinalYearIdx[i]]            
                t += 1       
            
            #   Identifying the tracers and the quality associated data items for
            #   the anchoring year.
            if AnchorYearIdx > -1:
                if (FirstYearIdx[i] > -1) and (FinalYearIdx[i] > -1):
                    AnchorTracer[i] = Tracer[i][AnchorYearIdx]
            
        #   Defining the gaps between the tracers and the default anchors. Either
        #   0 or 100 depending on whether the year is before or after the anchor
        #   year.
            if AnchorYearIdx > -1:
                t = 0
                for year in GBRange(DB_FIRSTYEAR, DB_FINALYEAR):
                    if (FirstYearIdx[i] > -1) and (FinalYearIdx[i] > -1):
                        GapTracer[i][t] = AnchorTracer[i]

                        if Tracer[i][t] >= Tracer[i][AnchorYearIdx]:
                            GapTracer[i][t] = 100 - AnchorTracer[i]

                        if Tracer[i][t] > 0:
                            DeltaTracer[i][t] = Tracer[i][t] - Tracer[i][AnchorYearIdx]

                        for key, record in Data[i].items():
                            record['Gap'][t] = record['Anchor']
                            if Tracer[i][t] >= Tracer[i][AnchorYearIdx]:
                                record['Gap'][t] = 100 - record['Anchor']

                            record['Ratio'][t] = 0
                            if GapTracer[i][t] > 0:
                                record['Ratio'][t] = record['Gap'][t] / GapTracer[i][t]

                            if Tracer[i][t] > 0:
                                record['Quality'][t] = record['Anchor'] + record['Ratio'][t] * DeltaTracer[i][t]
                    t += 1 

        #   ======================================================================
        #                    Set the calculated Quality into LiST
        #   ======================================================================

            if (FirstYearIdx[i] > -1) and (FinalYearIdx[i] > -1):
                for key, record in Data[i].items():
                    for t in GBRange(0, YearRec['ProjFinalYearIdx']):
                        dataYearIdx = 0

                        if YearRec['ProjFirstYear'] + t > DB_FIRSTYEAR:
                            dataYearIdx = YearRec['ProjFirstYear'] + t - DB_FIRSTYEAR

                        if dataYearIdx > NUM_DB_YEARS - 1:
                            dataYearIdx = NUM_DB_YEARS - 1

                        Coverage['quality'][record['currID']][t] = record['Quality'][dataYearIdx]

                        
                        UtilMstID = -1
                        if   IVInfo[record['mstID']]['QualityMode'] == CS_ANC1Mode: UtilMstID = CS_IV_ANC1_MST_ID
                        elif IVInfo[record['mstID']]['QualityMode'] == CS_ANC4Mode: UtilMstID = CS_IV_ANC4_MST_ID
                        elif IVInfo[record['mstID']]['QualityMode'] == CS_FascDelivMode: UtilMstID = CS_IV_HealthFacDelivery_MST_ID

                        if UtilMstID in IVInfo:
                            Utilization = Coverage['value'][IVInfo[UtilMstID]['currID']][t] / 100
                            value = ((record['Quality'][dataYearIdx] / 100) * Utilization) * 100
                            Coverage['value'][record['currID']][t] = value

    #   For now, KMC Quality will be zero even though there is readiness data
    #   in the database.  (Ticket #590, 6.14.22)
        for t in GBRange(0, YearRec['ProjFinalYearIdx']):
            Coverage['quality'][IVInfo[CS_IV_KangarooMothCare_MST_ID]['currID']][t] = 0
            Coverage['value'][IVInfo[CS_IV_KangarooMothCare_MST_ID]['currID']][t] = 0

    return Coverage

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                           Z Scores                                                       /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def CSGetAgeWeight(a):
    if   a == CS_0t1months   : return (1 - 0) / 60
    elif a == CS_1t6months   : return (6 - 1) / 60
    elif a == CS_6t12months  : return (12 - 6) / 60
    elif a == CS_12t24months : return (24 - 12) / 60
    elif a == CS_24t60months : return (60 - 24) / 60
    else                     : return 1

# Formulae are from "Stunting and Wasting Indicators 7aug2015.xlsx", cells D8 to D11.
def CSZScoreCalc(curYrSingle, firstYrSingle, firstYrGT1SD, firstYr1t2SD, firstYr2t3SD, firstYrGT3SD, 
                 valueGT1SD, value1t2SD, value2t3SD, valueGT3SD):

    if curYrSingle > firstYrSingle:
        temp = max(curYrSingle, 2.275)

        if curYrSingle > 2.275: 
            temp2 = 0
        else: 
            temp2 = 2.275 - curYrSingle

        temp3 = (firstYrSingle - temp) / (firstYrSingle - 2.275)

        valueGT1SD = firstYrGT1SD - (temp3 * (firstYrGT1SD - 84.1345))
        value1t2SD = firstYr1t2SD - (temp3 * (firstYr1t2SD - 13.5905)) + temp2
        value2t3SD = firstYr2t3SD - (temp3 * (firstYr2t3SD - 2.14)) - (temp2 / 2.275) * 2.14
        valueGT3SD = firstYrGT3SD - (temp3 * (firstYrGT3SD  - 0.135)) - (temp2 / 2.275) * 0.135
    else:
        temp = (curYrSingle - firstYrSingle) / (100 - firstYrSingle)

        valueGT1SD = firstYrGT1SD + temp * (0.13498 - firstYrGT1SD)
        value1t2SD = firstYr1t2SD + temp * (2.14 - firstYr1t2SD)
        value2t3SD = firstYr2t3SD + temp * (13.59046 - firstYr2t3SD)
        valueGT3SD = firstYrGT3SD + temp * (84.13453 - firstYrGT3SD)

def CSConvertIndicatorToZScores(FirstYrIdx, FinalYrIdx, distr, singleDistr):
    if FirstYrIdx >= len(singleDistr):
        exit

    firstYrSingle  = singleDistr[FirstYrIdx]
    
    for t in GBRange(FirstYrIdx, FinalYrIdx):
        curYrSingle = singleDistr[t]
        
        for a in GBRange(CS_0t1months, CS_24t60months):
            firstYrGT1SD = distr[CS_GT1STD][a][FirstYrIdx]
            firstYr1t2SD = distr[CS_1t2STD][a][FirstYrIdx]
            firstYr2t3SD = distr[CS_2t3STD][a][FirstYrIdx]
            firstYrGT3SD = distr[CS_GT3STD][a][FirstYrIdx]

            valueGT1SD = 0
            value1t2SD = 0
            value2t3SD = 0
            valueGT3SD = 0
            
            CSZScoreCalc(curYrSingle, firstYrSingle, firstYrGT1SD, firstYr1t2SD,
                firstYr2t3SD, firstYrGT3SD, valueGT1SD, value1t2SD, value2t3SD,
                valueGT3SD)

            distr[CS_GT1STD][a][t] = valueGT1SD
            distr[CS_1t2STD][a][t] = value1t2SD
            distr[CS_2t3STD][a][t] = value2t3SD
            distr[CS_GT3STD][a][t] = valueGT3SD

def CSConvertZScoresToIndicator(FirstYrIdx, FinalYrIdx, distr, singleDistr):
    for t in GBRange(FirstYrIdx, FinalYrIdx):
        singleDistr[t] = 0.0

        for a in GBRange(CS_0t1months, CS_24t60months):
            singleDistr[t] = singleDistr[t] + (distr[CS_2t3STD][a][t] + distr[CS_GT3STD][a][t]) * CSGetAgeWeight(a)

def CSUpdateDistributions(FirstYrIdx, FinalYrIdx, UseDetailedDistr, distr, singleDistr):
    # Calculate stunting to fill proper ModVars. If single stunting
    # is selected in the radio groups, then calculate detailed values. If detailed
    # stunting is selected in the radio groups, then calculate single values
    if UseDetailedDistr: 
        CSConvertZScoresToIndicator(FirstYrIdx, FinalYrIdx, distr, singleDistr)
    else: 
        CSConvertIndicatorToZScores(FirstYrIdx, FinalYrIdx, distr, singleDistr)

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def getCurrID(IVInfo, mstID) : 
    if mstID in IVInfo:
        return IVInfo[mstID]['currID']
    return None

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def StuntWasteDistr_RegionalInit(RegionData_DB, final_year_idx, tag):
    result = Value_1DSource_Init(np.zeros((CS_GT3STD+1, CS_24t60months+1, final_year_idx+1)).tolist())
    
    sourceData = RegionData_DB[tag]

    def fillAgeData(t, a, key):
        result['value'][CS_GT1STD][a][t] = sourceData[key]['>-1 Z']
        result['value'][CS_1t2STD][a][t] = sourceData[key]['-2 to -1Z']
        result['value'][CS_2t3STD][a][t] = sourceData[key]['-3 to -2 Z']
        result['value'][CS_GT3STD][a][t] = sourceData[key]['<-3 Z']

    for t in GBRange(0, final_year_idx): 
        fillAgeData(t, CS_0t1months, '0-1  month')
        fillAgeData(t, CS_1t6months, '1-5 months')
        fillAgeData(t, CS_6t12months, '6-11 months')
        fillAgeData(t, CS_12t24months, '12-23 months')
        fillAgeData(t, CS_24t60months, '24-59 months')

    Fill_Source_DBC(result, sourceData, '0')   

    return result

def BFPrev_RegionalInit(RegionData_DB, final_year_idx):
    result = Value_1DSource_Init(np.zeros((CS_NotBF+1, CS_24t60months+1, final_year_idx+1)).tolist())
       
    sourceData = RegionData_DB[TG_BreastFeeding]
    
    def fillAgeData(t, bf, a, key, key2):
        if key2 in sourceData[key]:
            result['value'][bf][a][t] = sourceData[key][key2]
    
    def fillBFData(t, bf, keySet):
        for key in keySet:
            fillAgeData(t, bf, CS_0t1months, key, 'less than 1')
            fillAgeData(t, bf, CS_1t6months, key, '1 to 5')
            fillAgeData(t, bf, CS_6t12months, key, '6 to 11')
            fillAgeData(t, bf, CS_12t24months, key, '12 to 23')

    for t in GBRange(0, final_year_idx):
        fillBFData(t, CS_ExclusiveBF, ['Exclusive breastfeeding'])
        fillBFData(t, CS_PredominantBF, ['Predominant breastfeeding'])
        fillBFData(t, CS_PartialBF, ['Partial breastfeeding', 'Any breastfeeding'])
        fillBFData(t, CS_NotBF, ['Not breastfeeding'])   
         
    Fill_Source_DBC(result, sourceData, '0')      
    
    return result

def SetRegionalValues(CSModvars, RegionData_DB, final_year_idx):
    hasStunting = False
    hasWasting = False
    hasBreastFeed = False

    for sd in GBRange(CS_GT1STD, CS_GT3STD):
        for age in GBRange(CS_AgeSummary, CS_24t60months):
            for t in GBRange(0, final_year_idx):
                if not CSModvars[CS_StuntDistrTag]['value'][sd][age][t] == 0:
                    hasStunting = True

    for sd in GBRange(CS_GT1STD, CS_GT3STD):
        for age in GBRange(CS_AgeSummary, CS_24t60months):
            for t in GBRange(0, final_year_idx):
                if not CSModvars[CS_WasteDistrTag]['value'][sd][age][t] == 0:
                    hasWasting = True

    for bf in GBRange(CS_ExclusiveBF, CS_NotBF):
        for age in GBRange(CS_AgeSummary, CS_24t60months):
            for t in GBRange(0, final_year_idx):
                if not CSModvars[CS_BFPrevTag]['value'][bf][age][t] == 0:
                    hasBreastFeed = True    

    if not hasStunting: CSModvars[CS_StuntDistrTag] = StuntWasteDistr_RegionalInit(RegionData_DB, final_year_idx, TG_StuntingDist)
    if not hasWasting: CSModvars[CS_WasteDistrTag] = StuntWasteDistr_RegionalInit(RegionData_DB, final_year_idx, TG_WastingDist)
    if not hasBreastFeed: CSModvars[CS_BFPrevTag] = BFPrev_RegionalInit(RegionData_DB, final_year_idx)

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def CSCalcCoverageRelationships(IVInfo, CalcArray, CalcSet, ReadingDatabase, scalingMethod = CS_SG_ScaleTargPerc):
    
    def sumValues(IVInfo, CalcArray, destMstID, sourceMstIDSet):    
        destCurrID = getCurrID(IVInfo, destMstID)
        if not destCurrID == None:
            CalcArray[destCurrID] = 0
            for mstID in sourceMstIDSet: 
                sourceCurrID = getCurrID(IVInfo, mstID)
                if not sourceCurrID == None:
                    CalcArray[destCurrID] += CalcArray[sourceCurrID]
    
    if CS_Pentavalent_CCFI in CalcSet:
        CS_IV_PentaVacc = getCurrID(IVInfo, CS_IV_PentaVacc_MST_ID)
        if not CS_IV_PentaVacc == None:
            
            CS_IV_DPT_Vacc = getCurrID(IVInfo, CS_IV_DPT_Vacc_MST_ID)
            if not CS_IV_DPT_Vacc == None: CalcArray[CS_IV_DPT_Vacc] = CalcArray[CS_IV_PentaVacc]
            
            CS_IV_HibVacc = getCurrID(IVInfo, CS_IV_HibVacc_MST_ID)
            if not CS_IV_HibVacc == None: CalcArray[CS_IV_HibVacc]  = CalcArray[CS_IV_PentaVacc]
            
            CS_IV_HepBVacc = getCurrID(IVInfo, CS_IV_HepBVacc_MST_ID)
            if not CS_IV_HepBVacc == None: CalcArray[CS_IV_HepBVacc] = CalcArray[CS_IV_PentaVacc]  
    
    if ReadingDatabase:
        CS_IV_InjectAntibNeonSeps = getCurrID(IVInfo, CS_IV_InjectAntibNeonSeps_MST_ID)
        CS_IV_ParenteralAdminOfAntibiotics = getCurrID(IVInfo, CS_IV_ParenteralAdminOfAntibiotics_MST_ID)   
        if not CS_IV_InjectAntibNeonSeps == None and not CS_IV_ParenteralAdminOfAntibiotics == None:
            CalcArray[CS_IV_InjectAntibNeonSeps] = CalcArray[CS_IV_ParenteralAdminOfAntibiotics]

        CS_IV_VitATrMeas = getCurrID(IVInfo, CS_IV_VitATrMeas_MST_ID)
        CS_IV_VitASupp = getCurrID(IVInfo, CS_IV_VitASupp_MST_ID)
        if not CS_IV_VitATrMeas == None and not CS_IV_VitASupp == None:
            CalcArray[CS_IV_VitATrMeas] = CalcArray[CS_IV_VitASupp]
    
    sumValues(IVInfo, CalcArray, CS_IV_CaseMgmtPrematBabies_MST_ID, [CS_IV_KangarooMothCare_MST_ID, CS_IV_FullSuppCarePremat_MST_ID, CS_IV_iKMC_MST_ID])
    sumValues(IVInfo, CalcArray, CS_IV_CaseMgmtNeonSepsPneu_MST_ID, [CS_IV_OralAntibNeonSeps_MST_ID, CS_IV_InjectAntibNeonSeps_MST_ID, CS_IV_FullSuppCareNeonSepsPneu_MST_ID])
    sumValues(IVInfo, CalcArray, CS_IV_MicronSupp_MST_ID, [CS_IV_MultiMicronSuppPreg_MST_ID, CS_IV_IronSuppPreg_MST_ID])
   
#  If scaling by percentage points instead of scaling to a target percentage 
#  (which the user may do, for example, in LiST SG), do some further checks. 
    if scalingMethod == CS_SG_ScalePercPts:
        CS_IV_MicronSupp = getCurrID(IVInfo, CS_IV_MicronSupp_MST_ID)
        CS_IV_CaseMgmtPrematBabies = getCurrID(IVInfo, CS_IV_CaseMgmtPrematBabies_MST_ID)
        CS_IV_CaseMgmtNeonSepsPneu = getCurrID(IVInfo, CS_IV_CaseMgmtNeonSepsPneu_MST_ID)
   
        if not CS_IV_MicronSupp == None:
            if CalcArray[CS_IV_MicronSupp] > 100.0: CalcArray[CS_IV_MicronSupp] = 100.0
            if CalcArray[CS_IV_MicronSupp] < -100.0: CalcArray[CS_IV_MicronSupp] = -100.0
        
        if not CS_IV_CaseMgmtPrematBabies == None:
            if CalcArray[CS_IV_CaseMgmtPrematBabies] > 100.0: CalcArray[CS_IV_CaseMgmtPrematBabies] = 100.0
            if CalcArray[CS_IV_CaseMgmtPrematBabies] < -100.0: CalcArray[CS_IV_CaseMgmtPrematBabies] = -100.0
        
        if not CS_IV_CaseMgmtNeonSepsPneu == None:
            if CalcArray[CS_IV_CaseMgmtNeonSepsPneu] > 100.0: CalcArray[CS_IV_CaseMgmtNeonSepsPneu] = 100.0
            if CalcArray[CS_IV_CaseMgmtNeonSepsPneu] < -100.0: CalcArray[CS_IV_CaseMgmtNeonSepsPneu] = -100.0

def CSRecalcDataForDataByCountry(CSModvars, final_year_idx):
    numIntervs = CSModvars[CS_NumIntervsTag]['value']
    
    # Recalc data for data by country
    for t in GBRange(0, final_year_idx):  
        OneDimCoverage = [0] * (numIntervs + 1)
        
        for i in GBRange(0, numIntervs):  
            OneDimCoverage[i] = CSModvars[CS_CoverageTag]['value'][i][t]
        
        CSCalcCoverageRelationships(CSModvars[CS_IVInfoTag], OneDimCoverage, CSModvars[CS_CalcCovFromInputsTag], True)
        
        for i in GBRange(0, numIntervs):  
            CSModvars[CS_CoverageTag]['value'][i][t] = OneDimCoverage[i]

def ModifyCoverage(CSModvars, final_year_idx):
    # Recalc data for data by country 
    CSRecalcDataForDataByCountry(CSModvars, final_year_idx)
  
    # Set BF improvement to exclusive breastfeeding value from health status  
    CS_IV_ChangesInBF = getCurrID(CSModvars[CS_IVInfoTag], str(CS_MstBFPromotion))
    if not CS_IV_ChangesInBF == None:         
        for t in GBRange(0, final_year_idx):  
            CSModvars[CS_CoverageTag]['value'][CS_IV_ChangesInBF][t] = CSModvars[CS_BFPrevTag]['value'][CS_ExclusiveBF][CS_1t6months][t]

    CSUpdateDistributions(0, final_year_idx, True, CSModvars[CS_StuntDistrTag]['value'], CSModvars[CS_SingleStuntDistrTag]['value'])
    CSUpdateDistributions(0, final_year_idx, True, CSModvars[CS_WasteDistrTag]['value'], CSModvars[CS_SingleWasteDistrTag]['value'])

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def CS_Linkage_HSNutriDefiOnChange(CSModvars):
    # Set affected fractions that rely on the HealthStatus variable.
    for a in GBRange(CS_AgeSummary, CS_24t60months):
        # Vitamin A
        value = CSModvars[CS_NutrDeficTag]['value'][CS_VitADefic]/100
        CSModvars[CS_DiarrIncReductionTag]['value'][CS_Default][CS_VitAPTwoDoses_DiarInterv][CS_AffecFract][a] = value
        CSModvars[CS_DiarrIncReductionTag]['value'][CS_Data][CS_VitAPTwoDoses_DiarInterv][CS_AffecFract][a] = value

        # Zinc
        value = CSModvars[CS_NutrDeficTag]['value'][CS_ZincDefic]/100
        CSModvars[CS_DiarrIncReductionTag]['value'][CS_Default][CS_ZincP_DiarInterv][CS_AffecFract][a] = value
        CSModvars[CS_DiarrIncReductionTag]['value'][CS_Data][CS_ZincP_DiarInterv][CS_AffecFract][a] = value

        # Zinc
        value = CSModvars[CS_NutrDeficTag]['value'][CS_ZincDefic]/100
        CSModvars[CS_PneumIncReductionTag]['value'][CS_Default][CS_ZincP_PneumInterv][CS_AffecFract][a] = value
        CSModvars[CS_PneumIncReductionTag]['value'][CS_Data][CS_ZincP_PneumInterv][CS_AffecFract][a] = value

        for d in GBRange(1, CS_MaxDeathCauses):
            # Vitamin A 
            CS_IV_VitASupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_VitASupp_MST_ID)
            if not CS_IV_VitASupp == None:
                value = CSModvars[CS_NutrDeficTag]['value'][CS_VitADefic]/100
                CSModvars[CS_ChildEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_VitASupp][d][a] = value
                CSModvars[CS_ChildEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_VitASupp][d][a] = value

            # Zinc
            CS_IV_ZincSupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_ZincSupp_MST_ID)
            if not CS_IV_ZincSupp == None:            
                value = CSModvars[CS_NutrDeficTag]['value'][CS_ZincDefic]/100
                CSModvars[CS_ChildEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_ZincSupp][d][a] = value
                CSModvars[CS_ChildEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_ZincSupp][d][a] = value

def CS_Linkage_HSMatNutriDefiOnChange(CSModvars):
    # Set affected fractions that rely on the HealthStatus variable.
    VitADefic = CSModvars[CS_VitADefInPWTag]['value'] / 100
    ZincDefic = CSModvars[CS_NutrDeficTag]['value'][CS_ZincDefic] / 100
    CalcDefic = CSModvars[CS_NutrDeficTag]['value'][CS_CalcDefic] / 100
    FolDefic  = CSModvars[CS_NutrDeficTag]['value'][CS_FolDefic] / 100
    IronDefic = CSModvars[CS_NutrDeficTag]['value'][CS_IronDefic] / 100

    for d in GBRange(1, CS_Mat_MaxDeathCauses):
        CS_IV_FolicAcidSupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_FolicAcidSupp_MST_ID)
        if not CS_IV_FolicAcidSupp == None:          
            CSModvars[CS_MatEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_FolicAcidSupp][d] = FolDefic
            CSModvars[CS_MatEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_FolicAcidSupp][d] = FolDefic

        CS_IV_ZincFort = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_ZincFort_MST_ID)
        if not CS_IV_ZincFort == None:          
            CSModvars[CS_MatEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_ZincFort][d] = ZincDefic
            CSModvars[CS_MatEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_ZincFort][d] = ZincDefic

        CS_IV_CalciumSupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_CalciumSupp_MST_ID)
        if not CS_IV_CalciumSupp == None:          
            CSModvars[CS_MatEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_CalciumSupp][d] = CalcDefic
            CSModvars[CS_MatEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_CalciumSupp][d] = CalcDefic

        CS_IV_ZincSuppInPreg = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_ZincSuppInPreg_MST_ID)
        if not CS_IV_ZincSuppInPreg == None:          
            CSModvars[CS_MatEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_ZincSuppInPreg][d] = ZincDefic
            CSModvars[CS_MatEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_ZincSuppInPreg][d] = ZincDefic
    
    FolicAcidDeficPrev = CSModvars[CS_FolicAcidDeficPrevTag]['value']
    
    for d in GBRange(1, CS_SB_MaxCauses):
        CS_IV_CalciumSupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_CalciumSupp_MST_ID)
        if not CS_IV_CalciumSupp == None:         
            CSModvars[CS_SBEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_CalciumSupp][d] = CalcDefic
            CSModvars[CS_SBEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_CalciumSupp][d] = CalcDefic

        CS_IV_FolicAcidSupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_FolicAcidSupp_MST_ID)
        if not CS_IV_FolicAcidSupp == None:          
            CSModvars[CS_SBEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_FolicAcidSupp][d] = FolDefic * FolicAcidDeficPrev
            CSModvars[CS_SBEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_FolicAcidSupp][d] = FolDefic * FolicAcidDeficPrev

    for d in GBRange(1, CS_MaxDeathCauses):
        value = FolDefic
        if d == CS_NNCongen:
            value = FolDefic * CSModvars[CS_CongenitalDueToNTDsAFTag]['value']
        
        for a in GBRange(CS_AgeSummary, CS_24t60months):
            CS_IV_FolicAcidSupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_FolicAcidSupp_MST_ID)
            if not CS_IV_FolicAcidSupp == None:       
                CSModvars[CS_ChildEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_FolicAcidSupp][d][a] = value
                CSModvars[CS_ChildEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_FolicAcidSupp][d][a] = value

            CS_IV_NNVitASupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_NNVitASupp_MST_ID)
            if not CS_IV_NNVitASupp == None:        
                CSModvars[CS_ChildEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_NNVitASupp][d][a] = VitADefic
                CSModvars[CS_ChildEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_NNVitASupp][d][a] = VitADefic

    for term in GBRange(CS_PreTermSGA, CS_TermAGA):
        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_CalciumSupp][term] = CalcDefic
        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_CalciumSupp][term] = CalcDefic

        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_FolicAcidFort][term] = FolDefic
        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_FolicAcidFort][term] = FolDefic

        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_ZincFort][term] = ZincDefic
        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_ZincFort][term] = ZincDefic

        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_ZincSuppInPreg][term] = ZincDefic
        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_ZincSuppInPreg][term] = ZincDefic

    for n in GBRange(CS_Preg, CS_NonPreg):
        CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Default][n][CS_IronFolatePW_MatAnemia][CS_AffecFract] = IronDefic
        CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Data][n][CS_IronFolatePW_MatAnemia][CS_AffecFract] = IronDefic

        CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Default][n][CS_IronFortNPW_MatAnemia][CS_AffecFract] = IronDefic
        CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Data][n][CS_IronFortNPW_MatAnemia][CS_AffecFract] = IronDefic

def CS_Linkage_HSFalciparumIPTpSheetOnChange(CSModvars):
    value = CSModvars[CS_PercExposedFalciparumTag]['value'] / 100

    CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Data][CS_Preg][CS_IPTP_MatAnemia][CS_AffecFract] = value
    CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Default][CS_Preg][CS_IPTP_MatAnemia][CS_AffecFract] = value

    CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Data][CS_Preg][CS_ITN_MatAnemia][CS_AffecFract] = value
    CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Default][CS_Preg][CS_ITN_MatAnemia][CS_AffecFract] = value

    CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Data][CS_NonPreg][CS_ITN_MatAnemia][CS_AffecFract] = value
    CSModvars[CS_ImpactOnMatAnemiaTag]['value'][CS_Default][CS_NonPreg][CS_ITN_MatAnemia][CS_AffecFract] = value

def CS_Linkage_HSPerCapitaIncOnChange(CSModvars):
    value = CSModvars[CS_FoodInsecureTag]['value'][CSModvars[CS_CovBYTag]['value']]
    if value == -1:
        value = CSModvars[CS_PerCapitaIncTag]['value'][CSModvars[CS_CovBYTag]['value']]

    value /= 100

    CS_IV_BalEnergySupp = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_BalEnergySupp_MST_ID)
    if not CS_IV_BalEnergySupp == None:        
        for d in GBRange(1, CS_Mat_MaxDeathCauses):
            CSModvars[CS_MatEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_BalEnergySupp][d] = value
            CSModvars[CS_MatEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_BalEnergySupp][d] = value
        
        for d in GBRange(1, CS_SB_MaxCauses):
            CSModvars[CS_SBEffTag]['value'][CS_Default][CS_AffecFract][CS_IV_BalEnergySupp][d] = value
            CSModvars[CS_SBEffTag]['value'][CS_Data][CS_AffecFract][CS_IV_BalEnergySupp][d] = value
    
    for term in GBRange(CS_PreTermSGA, CS_TermAGA):
        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_BalEnergy][term] = value
        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_BalEnergy][term] = value

def CS_Linkage_HSPercWom15t49LowBMIOnChange(CSModvars):
    # Set balanced energy affected fraction and stillbirth balanced energy
    # affected fraction = % of women aged 15-49 with low BMI.
    value = CSModvars[CS_PercWom15t49LowBMITag]['value']
    for term in GBRange(CS_PreTermSGA, CS_TermAGA):        
            CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_MultMicroLowBMI][term] = value
            CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_MultMicroLowBMI][term] = value

            CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_MultMicroNormBMI][term] = 1-value
            CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_MultMicroNormBMI][term] = 1-value

def CS_Linkage_HSSyphilisAmongWRAOnChange(CSModvars):
    # Set balanced energy affected fraction and stillbirth balanced energy
    # affected fraction = % of women aged 15-49 with low BMI.
    value = CSModvars[CS_SyphilisAmongWRATag]['value']
    for term in GBRange(CS_PreTermSGA, CS_TermAGA):        
            CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_SyphilisTreat][term] = value
            CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_SyphilisTreat][term] = value

def CS_Linkage_HSPathogensDiarrOnChange(CSModvars, defaultFlag):
    for age in GBRange(CS_0t1months, CS_24t60months):  
        # Incidence
        # Severe Diarrhea Rotavirus
        CSModvars[CS_SevDiarrIncReductionTag]['value'][defaultFlag][
            CS_RotavirusVacc_SevDiarInterv][CS_AffecFract][age] = CSModvars[CS_DiarrPathTag]['value'][
                CS_Distr_Sevr][CS_RotaPathDiar][age]
        
        # All Diarrhea Rotavirus
        CSModvars[CS_DiarrIncReductionTag]['value'][defaultFlag][
            CS_RotavirusVacc_DiarInterv][CS_AffecFract][age] = CSModvars[CS_DiarrPathTag]['value'][
                CS_Distr_All][CS_RotaPathDiar][age]
        
        # All Diarrhea Vaccine B
        CSModvars[CS_DiarrIncReductionTag]['value'][defaultFlag][
            CS_DiarVaccPathB_DiarInterv][CS_AffecFract][age] = CSModvars[CS_DiarrPathTag]['value'][
                CS_Distr_All][CS_PathBDiar][age]
        
        # All Diarrhea Vaccine C
        CSModvars[CS_DiarrIncReductionTag]['value'][defaultFlag][
            CS_DiarVaccPathC_DiarInterv][CS_AffecFract][age] = CSModvars[CS_DiarrPathTag]['value'][
                CS_Distr_All][CS_PathCDiar][age]
        
        # Detailed vaccine efficacy
        for dose in GBRange(1, CS_MaxVaccDose): 
            # Rota
            CSModvars[CS_DetVaccEffTag]['value'][defaultFlag][CS_AffecFract][
                CS_Rota_Det][CS_Diarrhea][dose][age] = CSModvars[CS_DiarrPathTag]['value'][
                CS_Distr_Fatal][CS_RotaPathDiar][age]
            
            # Vacc B
            CSModvars[CS_DetVaccEffTag]['value'][defaultFlag][CS_AffecFract][
                CS_VaccB_Det][CS_Diarrhea][dose][age] = CSModvars[CS_DiarrPathTag]['value'][
                CS_Distr_Fatal][CS_PathBDiar][age]
            
            # Vacc C
            CSModvars[CS_DetVaccEffTag]['value'][defaultFlag][CS_AffecFract][
                CS_VaccC_Det][CS_Diarrhea][dose][age] = CSModvars[CS_DiarrPathTag]['value'][
                CS_Distr_Fatal][CS_PathCDiar][age]
            
        # Standard vaccine efficacy
        # Rota
        CS_IV_RotavirusVacc = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_RotavirusVacc_MST_ID)
        if not CS_IV_RotavirusVacc == None:        
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_RotavirusVacc][CS_Diarrhea][age] = CSModvars[CS_DiarrPathTag]['value'][
                    CS_Distr_Fatal][CS_RotaPathDiar][age]
        
        # Vacc B
        CS_IV_DiarVaccPathB = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_DiarVaccPathB_MST_ID)
        if not CS_IV_DiarVaccPathB == None:        
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_DiarVaccPathB][CS_Diarrhea][age] = CSModvars[CS_DiarrPathTag]['value'][
                    CS_Distr_Fatal][CS_PathBDiar][age]

        # Vacc C
        CS_IV_DiarVaccPathC = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_DiarVaccPathC_MST_ID)
        if not CS_IV_DiarVaccPathC == None:        
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_DiarVaccPathC][CS_Diarrhea][age] = CSModvars[CS_DiarrPathTag]['value'][
                    CS_Distr_Fatal][CS_PathCDiar][age]

def CS_Linkage_HSPathogensPneumOnChange(CSModvars, defaultFlag):
    for age in GBRange(CS_0t1months, CS_24t60months):
        # Incidence
        # HiB
        CSModvars[CS_PneumIncReductionTag]['value'][
            defaultFlag][CS_HibVacc_PneumInterv][CS_AffecFract][age] = CSModvars[CS_PneumPathTag]['value'][
                CS_Distr_Sevr][CS_HibPathPneumVT][age]

        # PCV
        CSModvars[CS_PneumIncReductionTag]['value'][
            defaultFlag][CS_PneumVacc_PneumInterv][CS_AffecFract][age] = CSModvars[CS_PneumPathTag]['value'][
                CS_Distr_Sevr][CS_PneumocPathPneumVT][age]

        # Detailed vaccine efficacy
        for dose in GBRange(1, CS_MaxVaccDose):
            # HiB
            CSModvars[CS_DetVaccEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_HiB_Det][CS_Pneumonia][dose][age] = CSModvars[CS_PneumPathTag]['value'][
                    CS_Distr_Fatal][CS_HibPathPneumVT][age]

            # PCV
            CSModvars[CS_DetVaccEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_PCV_Det][CS_Pneumonia][dose][age] = CSModvars[CS_PneumPathTag]['value'][
                    CS_Distr_Fatal][CS_PneumocPathPneumVT][age]

        # Standard vaccine efficacy
        # HiB
        CS_IV_HibVacc = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_HibVacc_MST_ID)
        if not CS_IV_HibVacc == None:        
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_HibVacc][CS_Pneumonia][age] = CSModvars[CS_PneumPathTag]['value'][
                    CS_Distr_Fatal][CS_HibPathPneumVT][age]
        
        # PCV
        CS_IV_PneuVacc = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_PneuVacc_MST_ID)
        if not CS_IV_PneuVacc == None:        
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_PneuVacc][CS_Pneumonia][age] = CSModvars[CS_PneumPathTag]['value'][
                    CS_Distr_Fatal][CS_PneumocPathPneumVT][age]

def CS_Linkage_HSPathogensMeninOnChange(CSModvars, defaultFlag):
    for age in GBRange(CS_0t1months, CS_24t60months):
        # Incidence
        # HiB
        CSModvars[CS_MeninIncReductionTag]['value'][
            defaultFlag][CS_HibVacc_MeningInterv][CS_AffecFract][age] = CSModvars[CS_MeninPathTag]['value'][
                CS_Distr_Sevr][CS_HibPathMeninVT][age]

        # PCV
        CSModvars[CS_MeninIncReductionTag]['value'][
            defaultFlag][CS_PneumVacc_MeningInterv][CS_AffecFract][age] = CSModvars[CS_MeninPathTag]['value'][
                CS_Distr_Sevr][CS_PneumocPathMeninVT][age]
        
        # Meningococcal - A
        CSModvars[CS_MeninIncReductionTag]['value'][
            defaultFlag][CS_MeningA_MeningInterv][CS_AffecFract][age] = CSModvars[CS_MeninPathTag]['value'][
                CS_Distr_Sevr][CS_MeningitAPathMeninVT][age]

        # Detailed vaccine efficacy
        for dose in GBRange(1, CS_MaxVaccDose):
            # HiB
            CSModvars[CS_DetVaccEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_HiB_Det][CS_Meningitis][dose][age] = CSModvars[CS_MeninPathTag]['value'][
                    CS_Distr_Fatal][CS_HibPathMeninVT][age]
            
            # PCV
            CSModvars[CS_DetVaccEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_PCV_Det][CS_Meningitis][dose][age] = CSModvars[CS_MeninPathTag]['value'][
                    CS_Distr_Fatal][CS_PneumocPathMeninVT][age]
            
            # Meningococcal A
            CSModvars[CS_DetVaccEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_MenA_Det][CS_Meningitis][dose][age] = CSModvars[CS_MeninPathTag]['value'][
                    CS_Distr_Fatal][CS_MeningitAPathMeninVT][age]

        # Standard vaccine efficacy
        # HiB
        CS_IV_HibVacc = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_HibVacc_MST_ID)
        if not CS_IV_HibVacc == None:              
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_HibVacc][CS_Meningitis][age] = CSModvars[CS_MeninPathTag]['value'][
                    CS_Distr_Fatal][CS_HibPathMeninVT][age]
        
        # PCV
        CS_IV_PneuVacc = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_PneuVacc_MST_ID)
        if not CS_IV_PneuVacc == None:              
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_PneuVacc][CS_Meningitis][age] = CSModvars[CS_MeninPathTag]['value'][
                    CS_Distr_Fatal][CS_PneumocPathMeninVT][age]
        
        # Meningococcal A
        CS_IV_MeningA = getCurrID(CSModvars[CS_IVInfoTag], CS_IV_MeningA_MST_ID)
        if not CS_IV_MeningA == None:              
            CSModvars[CS_ChildEffTag]['value'][
                defaultFlag][CS_AffecFract][CS_IV_MeningA][CS_Meningitis][age] = CSModvars[CS_MeninPathTag]['value'][
                    CS_Distr_Fatal][CS_MeningitAPathMeninVT][age]

def CS_Linkage_EffIntervForBOOnChange(CSModvars):
    #  IPTp malaria and ITN will be calculated later. Zero out data for now.
    #  Affected fraction for IPT and ITN is %births that are first or second order
    #  times the % of women exposed to Falciparum (See CSProj for details).
    value = 0
    for term in GBRange(CS_PreTermSGA, CS_TermAGA):
        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_IPTp][term] = value
        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_IPTp][term] = value

        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_AspirinLowDose][term] = value
        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_AspirinLowDose][term] = value

        CSModvars[CS_IntervForBOTag]['value'][CS_Data][CS_AffecFract][CS_Progesterone][term] = value
        CSModvars[CS_IntervForBOTag]['value'][CS_Default][CS_AffecFract][CS_Progesterone][term] = value

def CS_Linkage_AFforKMConBreastfeedingOnChange(CSModvars, defaultFlag):
    # Set the Affected Fraction for KMC on Breastfeeding to 0.58 * percent of births that
    # are premature, which is a country-specific value.
    value = 0.58 * (CSModvars[CS_StatusAtBirthTag]['value'][CS_PreTermSGA] + CSModvars[CS_StatusAtBirthTag]['value'][CS_PreTermAGA])
    
    for a in GBRange(CS_0t1months, CS_24t60months):
        CSModvars[CS_KMConBFTag]['value'][defaultFlag][CS_AffecFract][a] = value

def ModifyEffectivenessFromHealthStatus(CSModvars):    
    CS_Linkage_HSNutriDefiOnChange(CSModvars)
    CS_Linkage_HSMatNutriDefiOnChange(CSModvars)
    CS_Linkage_HSFalciparumIPTpSheetOnChange(CSModvars)
    CS_Linkage_HSPerCapitaIncOnChange(CSModvars)
    CS_Linkage_HSPercWom15t49LowBMIOnChange(CSModvars)
    CS_Linkage_HSSyphilisAmongWRAOnChange(CSModvars)
    CS_Linkage_HSPathogensDiarrOnChange(CSModvars, CS_Default)
    CS_Linkage_HSPathogensPneumOnChange(CSModvars, CS_Default)
    CS_Linkage_HSPathogensMeninOnChange(CSModvars, CS_Default)
    CS_Linkage_HSPathogensDiarrOnChange(CSModvars, CS_Data)
    CS_Linkage_HSPathogensPneumOnChange(CSModvars, CS_Data)
    CS_Linkage_HSPathogensMeninOnChange(CSModvars, CS_Data)
    CS_Linkage_EffIntervForBOOnChange(CSModvars)
    CS_Linkage_AFforKMConBreastfeedingOnChange(CSModvars, CS_Default)
    CS_Linkage_AFforKMConBreastfeedingOnChange(CSModvars, CS_Data)

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# /////                                                                                                          /////
# /////                                                                                                          /////
# /////                                                                                                          /////
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

def setBaseYrValueAtYr(arrPtr, t):
    arrPtr[t] = arrPtr[0]

def FlatlineCoverage(CSModvars, final_year_idx):
    for t in GBRange(0, final_year_idx):
        for i in GBRange(0, len(CSModvars[CS_CoverageTag]['value']) - 1):
            setBaseYrValueAtYr(CSModvars[CS_CoverageTag]['value'][i], t)
                
        for bf in GBRange(0, CS_NotBF):
            for a in GBRange(0, CS_24t60months):
                setBaseYrValueAtYr(CSModvars[CS_BFPrevTag]['value'][bf][a], t)
        
        for a in GBRange(0, CS_24t60months):
            setBaseYrValueAtYr(CSModvars[CS_EarlyInitBFPrevTag]['value'][a], t)
        
        for vt in GBRange(0, CS_LastVacc_Det):
            for dose in GBRange(0, CS_MaxVaccDose):
                setBaseYrValueAtYr(CSModvars[CS_RoutDetVaccCovTag]['value'][vt][dose], t)
        
        setBaseYrValueAtYr(CSModvars[CS_PentaDetVaccCovTag]['value'], t)
        
        for sd in GBRange(0, CS_GT3STD):
            for a in GBRange(0, CS_24t60months): 
                setBaseYrValueAtYr(CSModvars[CS_StuntDistrTag]['value'][sd][a], t)
                setBaseYrValueAtYr(CSModvars[CS_WasteDistrTag]['value'][sd][a], t)
    
    for t in GBRange(CS_DetVacc1YrsRetro, CS_DetVaccRetroOffset):
        for vt in GBRange(0, CS_LastVacc_Det):
            for dose in GBRange(0, CS_MaxVaccDose):
                CSModvars[CS_RoutDetVaccCovTag]['retro'][vt][dose][t] = CSModvars[CS_RoutDetVaccCovTag]['retro'][vt][dose][CS_DetVacc1YrsRetro]
        CSModvars[CS_PentaDetVaccCovTag]['retro'][t] = CSModvars[CS_PentaDetVaccCovTag]['retro'][CS_DetVacc1YrsRetro]

def GetStandardCoverage(CSModvars, mstID, t):
    mstID = int(mstID)
    IVInfo = CSModvars[CS_IVInfoTag]

    result = 0

    if str(mstID) in IVInfo:
        if 'currID' in IVInfo[str(mstID)]:
            currID = IVInfo[str(mstID)]['currID']
            vt = CSGetVaccCurrID(mstID)
            if not vt == 0:
                dose = CSGetDetVaccEquivDose(vt)
                result = CSModvars[CS_RoutDetVaccCovTag]['value'][vt][dose][t]
            elif mstID == IV_MstPentaVacc:
                result = CSModvars[CS_PentaDetVaccCovTag]['value'][t]
            else:
                result = CSModvars[CS_CoverageTag]['value'][currID][t]
    
    return result

def SetStandardCoverage(CSModvars, mstID, t, value):
    mstID = int(mstID)
    IVInfo = CSModvars[CS_IVInfoTag]

    if str(mstID) in IVInfo:
        if 'currID' in IVInfo[str(mstID)]:
            currID = IVInfo[str(mstID)]['currID']
            vt = CSGetVaccCurrID(mstID)
            if not vt == 0:
                dose = CSGetDetVaccEquivDose(vt)
                CSModvars[CS_RoutDetVaccCovTag]['value'][vt][dose][t] = value
            elif mstID == IV_MstPentaVacc:
                CSModvars[CS_PentaDetVaccCovTag]['value'][t] = value
            else:
                CSModvars[CS_CoverageTag]['value'][currID][t] = value

def SetNonStandardCoverage(CSModvars, mstID, t, value):
    mstID = int(mstID)
    
    var1, var2 = getVars(mstID)

    if   mstID in BF_Set    : CSModvars[CS_BFPrevTag]['value'][var1][var2][t] = value
    elif mstID in Stunt_Set : CSModvars[CS_StuntDistrTag]['value'][var1][var2][t] = value
    elif mstID in Waste_Set : CSModvars[CS_WasteDistrTag]['value'][var1][var2][t] = value
    
    elif mstID == CS_MstEarlyInitBF : 
        for a in GBRange(CS_AgeSummary, CS_24t60months):     
            CSModvars[CS_EarlyInitBFPrevTag]['value'][a][t] = value

def UpdateCoverageFromQuality(CSModvars, final_year_idx):
    IVInfo = CSModvars[CS_IVInfoTag]

    for key, record in IVInfo.items():
        if 'currID' in IVInfo[key]:
            currID = IVInfo[key]['currID']
            qualityMode = IVInfo[key]['QualityMode']
            
            utilMstID = -1
            if not qualityMode == CS_QualityInactive:
                if qualityMode == CS_ANC1Mode: utilMstID = CS_IV_ANC1_MST_ID
                if qualityMode == CS_ANC4Mode: utilMstID = CS_IV_ANC4_MST_ID
                if qualityMode == CS_FascDelivMode: utilMstID = CS_IV_HealthFacDelivery_MST_ID

                if utilMstID in IVInfo:
                    
                    utilCurrID = IVInfo[utilMstID]['currID']
                    
                    for t in GBRange(0, final_year_idx):
                        Utilization = CSModvars[CS_CoverageTag]['value'][utilCurrID][t]
                        Quality = CSModvars[CS_CoverageTag]['quality'][currID][t]
                        CSModvars[CS_CoverageTag]['value'][currID][t] = (Quality / 100) * Utilization
                    
                    if int(utilMstID) in CSModvars[CS_SubnatDefDataMstIDsetTag]['value']:
                        CSModvars[CS_SubnatDefDataMstIDsetTag]['value'].append(int(key))
                        if utilMstID in CSModvars[CS_SubnatDataSourceArrayTag]['value']:
                            CSModvars[CS_SubnatDataSourceArrayTag]['value'][key] = CSModvars[CS_SubnatDataSourceArrayTag]['value'][utilMstID]

def SetInterventionsActive(CSModvars, interventionsToExclude):
    '''
        Sets all interventions active except any in interventionsToExclude

            Parameters:
                CSModvars (dict): MV tag, MV value pairs of all MVs required to execute this function.  The following MVs are required:

                    CS_IVInfoTag
                    IC_CS_SelectedIVSetTag
                    CS_EnterBFImproveDataTag

                interventionsToExclude (list) : list of all interventions that should NOT be made active.
              
            Returns:
                Nothing.
    '''

    for key, record in CSModvars[CS_IVInfoTag].items():
        if key not in [CS_IV_ContraceptiveUse_MST_ID, 
                       CS_IV_ART_MST_ID, 
                       str(CS_MstBFPromotion)]:
            if "Coverage_ED" in record:
                if record["Coverage_ED"]: 
                    if (int(key) not in interventionsToExclude) and (int(key) not in CSModvars[CS_SelectedIVSetTag]):             
                        CSModvars[CS_SelectedIVSetTag].append(int(key))
    
    for mstID in GBRange(CS_FirstFPMstID, CS_LastFPMstID):
        if (mstID not in interventionsToExclude) and (mstID not in CSModvars[CS_SelectedIVSetTag]):
            CSModvars[CS_SelectedIVSetTag].append(mstID)
    
    if (CS_FirstAMMstID not in interventionsToExclude) and (CS_FirstAMMstID not in CSModvars[CS_SelectedIVSetTag]):
        CSModvars[CS_SelectedIVSetTag].append(CS_FirstAMMstID)  

    if (CS_MstPercOnART not in interventionsToExclude) and (CS_MstPercOnART not in CSModvars[CS_SelectedIVSetTag]):
        CSModvars[CS_SelectedIVSetTag].append(CS_MstPercOnART)

    # Breastfeeding promotion needs to be active for IHT on projection creation.
    if (CS_MstBFPromotion not in interventionsToExclude) and (CS_MstBFPromotion not in CSModvars[CS_SelectedIVSetTag]):
        CSModvars[CS_SelectedIVSetTag].append(CS_MstBFPromotion)
    # CSModvars[CS_EnterBFImproveDataTag] = CS_MstBFPromotionMode


    





from SpectrumCommon.Const.IC import *
import SpectrumCommon.Const.IC.ICTags as icmvt
from SpectrumCommon.Modvars.GB.GBDefs import *

ic_interv_dim = 'ic_interv'
ic_interv_indices = {
    
}

ic_targ_pop_dim = 'ic_targ_pop'
ic_targ_pop_indices = {}

ic_deliv_channel_dim = 'ic_deliv_channel'
ic_deliv_channel_indices = {
    1: "Community",
    2: "Outreach",
    5: "Prehospital emergency",
    6: "General outpatient services",
    4: "First referral",
    7: "Second referral and above",
    
}

ic_visit_type_dim = 'ic_visit_type'
ic_visit_type_indices = {
    IC_OUTPATIENT_VISIT_CURR_ID : 'Outpatient visit',
    IC_INPATIENT_DAY_CURR_ID    : 'Inpatient day'
}

ic_treat_input_type_dim = 'ic_treat_input_type'
ic_treat_input_type_indices = {
}

ic_drug_supply_dim = 'ic_drug_supply'
ic_drug_supply_indices = {
}

ic_med_personnel_dim = 'ic_med_personnel'
ic_med_personnel_indices = {
}

ic_costed_deliv_chan_dim = 'ic_costed_deliv_chan'
ic_costed_deliv_chan_ids = {}

ic_staff_dim = 'ic_staff'
ic_staff_indices = {}   

ic_prog_area_dim = 'ic_prog_area'
ic_prog_area_indices = {}

ic_group_set_1_dim = 'ic_group_set_1'
ic_group_set_1_indices = { 
    IC_AGE_GROUP_1_LT_50_CURR_ID    : 'Less than 50',
    IC_AGE_GROUP_1_50_TO_59_CURR_ID : '50 to 59',
    IC_AGE_GROUP_1_60_TO_69_CURR_ID : '60 to 69',
    IC_AGE_GROUP_1_70_PLUS_CURR_ID  : '70+',
}
ic_group_set_2_dim = 'ic_group_set_2'
ic_group_set_2_indices = {

# Age group 2: 0-4, 5-14, 15-29, 30-44, 45-59, 60-69, 70-79, 80+
    IC_AGE_GROUP_2_0_TO_4_CURR_ID    : '0 to 4',
    IC_AGE_GROUP_2_5_TO_14_CURR_ID   : '5 to 14',
    IC_AGE_GROUP_2_15_TO_29_CURR_ID  : '15 to 29',
    IC_AGE_GROUP_2_30_TO_44_CURR_ID  : '30 to 44',  
    IC_AGE_GROUP_2_45_TO_59_CURR_ID  : '45 to 59',
    IC_AGE_GROUP_2_60_TO_69_CURR_ID  : '60 to 69',
    IC_AGE_GROUP_2_70_TO_79_CURR_ID  : '70 to 79',
    IC_AGE_GROUP_2_80_PLUS_CURR_ID   : '80+',
}

IC_modvar_shapes = {
        icmvt.IC_VERSION_TAG_V1                                   : gb_int_shape('IC Version'),
        icmvt.IC_DATA_CHANGED_TAG_V1                              : gb_bool_shape('IC Data Changed'), 
        icmvt.IC_PROJ_VALID_TAG_V1                                : gb_bool_shape('IC Projection Valid'), 
        icmvt.IC_PROJ_VALID_DATE_TAG_V1                           : gb_str_shape('IC Projection Valid Date'),  
        icmvt.IC_SHOW_ALL_YEARS_INTERV_COV_TAG_V1                 : gb_bool_shape('IC Show All Years Intervention Coverage'), 
        icmvt.IC_SHOW_ALL_FP_METHODS_INTERV_COV_TAG_V1            : gb_bool_shape('IC Show All Years FP Methods Intervention Coverage'), 
        icmvt.IC_SHOW_ALL_YEARS_INTERV_PIN_TAG_V1                 : gb_bool_shape('IC Show All Years Intervention PIN'), 
        icmvt.IC_ENTER_DET_PMTCT_DATA_TAG_V1                      : gb_bool_shape('IC Enter Detailed PMTCT Data'), 
        icmvt.IC_ENTER_NET_AGE_DIST_PERC_TAG_V1                   : gb_bool_shape('IC Enter Net Age Distribution as Percentages'), 
        icmvt.IC_POST_TREAT_SURV_FOLLOW_UP_VIS_TAG_V1             : gb_bool_shape('IC Post Treatment Survey Follow Up Visited'), 
        icmvt.IC_MAX_CUSTOM_DRUG_SUPP_MST_ID_TAG_V1               : gb_int_shape('IC Max Custom Drug Supply Master ID'), 
        icmvt.IC_COV_YEAR_BEF_BASE_YEAR_ITN_TAG_V1                : gb_float_shape('IC Coverage Year Before Base Year ITN'),
        icmvt.IC_NET_LIFETIME_ITN_TAG_V1                          : gb_float_shape('IC Net Lifetime ITN'),
        icmvt.IC_AVG_NET_AGE_ITN_TAG_V1                           : gb_float_shape('IC Average Net Age ITN'),
        icmvt.IC_INTERV_RECORDS_TAG                               : gb_shape('IC Intervention Records', '1D dict',
                                                                             dim=[ic_interv_dim],
                                                                             indices=[ic_interv_indices],
                                                                             innerShape= {
                                                                                 
                                                                                 'intervID':gb_str_shape('Intervention ID'), 
                                                                                 'name':gb_str_shape('Intervention Name'), 
                                                                                 'stringConstant':gb_str_shape('Intervention String Constant'), 
                                                                                 'active':gb_bool_shape('Is active'), 
                                                                                 'custom':gb_bool_shape('Is custom intervention'), 
                                                                                 'impact':gb_bool_shape('Is impact intervention'), 
                                                                                 'progAreaID':gb_str_shape('Program Area ID'), 
                                                                                 'subgroupID':gb_str_shape('Subgroup ID'),
                                                                                 'specialCases':gb_shape('Special Cases - contains tags for special use', '1D str'), 
                                                                                 'stratifying':gb_bool_shape('Is stratifying intervention'), 
                                                                                 'costForPHC':gb_bool_shape('Is cost for PHC'), 
                                                                                 'PIN_EditorVisited':gb_bool_shape('PIN Editor Visited'), 
                                                                                 'coverageChanged':gb_bool_shape('Coverage Changed'), 
                                                                                 'PIN_StringConstant':gb_str_shape('PIN String Constant'), 
                                                                                 'coverage':gb_year_shape('Coverage'), # This is a percentage, so it is a float.
                                                                                 'treatInputNotReq' : gb_shape('Treatment Input Not Required', '2D float', 
                                                                                                               dim=[ic_deliv_channel_dim, ic_treat_input_type_dim],
                                                                                                               indices=[ic_deliv_channel_indices, ic_treat_input_type_indices]),
                                                                                 'maxCustomTIGroupMstID': gb_shape('max custom TI group master id', '2D int', 
                                                                                                               dim=[ic_deliv_channel_dim, ic_treat_input_type_dim],
                                                                                                               indices=[ic_deliv_channel_indices, ic_treat_input_type_indices]),
                                                                                 'PIN':gb_year_shape('Population in need'), # This is a percentage, so it is a float.
                                                                                 'percDelivThroughChan':gb_shape('Percent delivery through channel', '2D float', 
                                                                                                               dim=[ic_deliv_channel_dim, 'ic_base_targ_dim'],
                                                                                                               indices=[ic_deliv_channel_indices, {}]),
                                                                                 'visitCostPerCase':gb_shape('Vist cost per case', '3D float', 
                                                                                                               dim=[ic_visit_type_dim, ic_deliv_channel_dim, gb_year_dim],
                                                                                                               indices=[ic_visit_type_indices, ic_deliv_channel_indices, gb_year_indices],
                                                                                                               contingent=True), # This is a percentage, so it is a float.
                                                                                 'targPopRec':gb_shape('Target Population Records', 'dict',
                                                                                                        innerShape={
                                                                                                                'targPopMstID': gb_str_shape('Target Population ID'),
                                                                                                                'sex': gb_int_shape('sex id'),
                                                                                                                'startAge':gb_int_shape('Start Age'),
                                                                                                                'endAge':gb_int_shape('End Age'),
                                                                                                                'directEntryValues':gb_year_shape('Direct Entry Values')
                                                                                                            })
                                                                                },
                                                                             has_default=False,
                                                                             contingent=True, # Contingent upon filling out this MV shape based on the file above in a maintainable way.
                                                                             ),
        icmvt.IC_TREAT_INPUT_DRUG_SUPPLY_RECORDS_TAG              : gb_shape('Drug and supply records', '1D dict',
                                                                             dim=[ic_drug_supply_dim],
                                                                             indices=[ic_drug_supply_indices],
                                                                             innerShape= {
                                                                                 'intervID':gb_str_shape('Intervention ID'), 
                                                                                 'delivChanID': gb_str_shape('Delivery Channel ID'), 
                                                                                 'treatInputGroupID':gb_str_shape('Treatment Input Group ID'), 
                                                                                 'treatInputID':gb_str_shape('Treatment Input ID'), 
                                                                                 'percApplied':gb_year_shape('Percentage Applied'), # This is a percentage, so it is a float.
                                                                                 'note':gb_str_shape('Note'), 
                                                                                 'stringConstant':gb_str_shape('String Constant'), 
                                                                                 'timesPerDay': gb_float_shape('Times per Day'), 
                                                                                 'daysPerCase': gb_float_shape('Days per Case'), 
                                                                                 'number': gb_float_shape('Number'),
                                                                             },
                                                                             has_default=False
                                                                    ),
        icmvt.IC_TREAT_INPUT_MED_PERSONNEL_RECORDS_TAG            : gb_shape('Med personnel records', '1D dict',
                                                                             dim=[ic_med_personnel_dim],
                                                                             indices=[ic_med_personnel_indices],
                                                                             innerShape= {
                                                                                 'intervID':gb_str_shape('Intervention ID'), 
                                                                                 'delivChanID': gb_str_shape('Delivery Channel ID'), 
                                                                                 'treatInputGroupID':gb_str_shape('Treatment Input Group ID'), 
                                                                                 'treatInputID':gb_int_shape('Treatment Input ID'),  # why int?
                                                                                 'percApplied':gb_year_shape('Percentage Applied'), # This is a percentage, so it is a float.
                                                                                 'note':gb_str_shape('Note'), 
                                                                                 'stringConstant':gb_str_shape('String Constant'), 
                                                                                 'customStaffType': gb_bool_shape('Custom Staff Type'),
                                                                                 'skilledStaffCost': gb_bool_shape('Skilled Staff Cost'),
                                                                                 'numOfDays': gb_float_shape('Number of days'), 
                                                                                 'minutes': gb_float_shape('Minutes'), 
                                                                             },
                                                                             has_default=False
                                                                    ),
        icmvt.IC_TREAT_INPUT_OUTPAT_INPAT_RECORDS_TAG             : gb_shape('Outpatient inpatient records', '1D dict',
                                                                             innerShape={
                                                                                 'intervID':gb_str_shape('Intervention ID'),
                                                                                 'delivChanID': gb_str_shape('Delivery Channel ID'),
                                                                                 'treatInputGroupID':gb_str_shape('Treatment Input Group ID'),
                                                                                 'treatInputID':gb_str_shape('Treatment Input ID'),
                                                                                 'percApplied':gb_year_shape('Percentage Applied'), # This is a percentage, so it is a float.
                                                                                 'note':gb_str_shape('Note'),
                                                                                 'stringConstant':gb_str_shape('String Constant'),
                                                                                 'unitsPerCase': gb_float_shape('Units per Case')
                                                                                 }
                                                                    ),
        icmvt.IC_TREAT_INPUT_GROUP_DRUG_SUPPLY_RECORDS_TAG        : gb_shape('IC Treatment Input Group Drug Supply Records', '1D dict',
                                                                             innerShape={
                                                                                 'intervID': gb_str_shape('Intervention ID'),
                                                                                    'delivChanID': gb_str_shape('Delivery Channel ID'),
                                                                                    'treatInputGroupID': gb_str_shape('Treatment Input Group ID'),
                                                                                    'name': gb_str_shape('Name'),
                                                                                    'stringConstant': gb_str_shape('String Constant'),
                                                                             }
                                                                    ),
        icmvt.IC_TREAT_INPUT_GROUP_MED_PERSONNEL_RECORDS_TAG      : gb_shape('IC Treatment Input Group Medical Personnel Records', '1D dict',
                                                                             innerShape={   
                                                                                    'intervID': gb_str_shape('Intervention ID'),
                                                                                    'delivChanID': gb_str_shape('Delivery Channel ID'),
                                                                                    'treatInputGroupID': gb_str_shape('Treatment Input Group ID'),
                                                                                    'name': gb_str_shape('Name'),
                                                                                    'stringConstant': gb_str_shape('String Constant')
                                                                             }
                                                                    ),
        icmvt.IC_TREAT_INPUT_GROUP_OUTPAT_INPAT_RECORDS_TAG       : gb_shape('IC Treatment Input Group Outpatient Inpatient Records', '1D dict',
                                                                                innerShape={
                                                                                    'intervID': gb_str_shape('Intervention ID'),
                                                                                    'delivChanID': gb_str_shape('Delivery Channel ID'), 
                                                                                    'treatInputGroupID': gb_str_shape('Treatment Input Group ID'),
                                                                                    'name': gb_str_shape('Name'),
                                                                                    'stringConstant': gb_str_shape('String Constant')
                                                                                }
                                                                    ),
        icmvt.IC_DRUG_SUPPLY_RECORDS_TAG                          : gb_shape('IC Drug Supply Records', '1D dict',
                                                                            dim=[ic_drug_supply_dim],
                                                                            indices=[ic_drug_supply_indices],
                                                                            innerShape={
                                                                                'drugSupplyID': gb_str_shape('Drug Supply ID'),
                                                                                'name': gb_str_shape('Name'),
                                                                                'stringConstant': gb_str_shape('String Constant'),
                                                                                'custom': gb_bool_shape('Is custom drug supply'),
                                                                                #'cost': gb_year_shape('Cost'),  
                                                                                'active': gb_bool_shape('Is active'),
                                                                                'costTypeMstID': gb_int_shape('Cost Type Master ID'),
                                                                                'unitCost': gb_year_shape('Unit Cost'),  
                                                                                'applicProgAreaIDs': gb_shape('Applicable Program Area ID', '1D str'),  
                                                                            }    
                                                                    ),                                     
        icmvt.IC_TARG_POP_RECORDS_TAG                             : gb_shape('IC Target Population Records', '1D dict',
                                                                            dim=[ic_targ_pop_dim],    
                                                                            indices=[ic_targ_pop_indices],
                                                                            innerShape={    
                                                                                'name': gb_str_shape('Target Population Name'),
                                                                                'stringConstant': gb_str_shape('Target Population String Constant'),
                                                                                'defTargPopMstID': gb_int_shape('Default Target Population Master ID'),
                                                                                'applicProgAreaIDs': gb_shape('Applicable Program Area ID', '1D str'),  
                                                                                'requiredModMstIDs' : gb_shape('Required modules', '1D str'),  
                                                                            }   
                                                                    ),
		icmvt.IC_ALT_PIN_AGE_GROUP_SET_1_LIST_TAG_V1              : gb_shape('IC Alternative PIN Age Group Set 1 List', '1D dict',
                                                                       innerShape={
                                                                           'intervID': gb_str_shape('Intervention ID'),
                                                                           'PIN' : gb_shape('PIN', '2D float',  
                                                                                      dim=[ic_group_set_1_dim, gb_year_dim],
                                                                                      indicies=[ic_group_set_1_indices, gb_year_indices]),
                                                                       }
                                                                    ),
        icmvt.IC_ALT_PIN_AGE_GROUP_SET_2_LIST_TAG_V1              : gb_shape('IC Alternative PIN Age Group Set 1 List', '1D dict',
                                                                       innerShape={
                                                                           'intervID': gb_str_shape('Intervention ID'),
                                                                           'PIN' : gb_shape('PIN', '2D float',  
                                                                                      dim=[ic_group_set_2_dim, gb_year_dim],
                                                                                      indicies=[ic_group_set_2_indices, gb_year_indices]),
                                                                       }
                                                                    ),
        icmvt.IC_VARY_TREATMENT_INPUTS_BY_TIME_TAG_V1             : gb_float_shape('IC Vary Treatment Inputs by Time'),
        icmvt.IC_DRUG_COST_PER_CASE_TAG_V1                        : gb_shape('IC Drug Cost per Case', '1D dict',
                                                                            dim=[ic_drug_supply_dim],
                                                                            indices=[ic_drug_supply_indices],
                                                                            innerShape={
                                                                                'intervID': gb_str_shape('Intervention ID'),
                                                                                'delivChanID': gb_str_shape('Delivery Channel ID'),
                                                                                'value': gb_year_shape('Value')  # This is a percentage, so it is a float.
                                                                            },
                                                                            has_default=False
                                                                        ),
                                                                             
        icmvt.IC_SUPPLY_COST_PER_CASE_TAG_V1                      : gb_shape('IC Supply Cost per Case', '1D dict',
                                                                            dim=[ic_drug_supply_dim],
                                                                            indices=[ic_drug_supply_indices],
                                                                            innerShape={
                                                                                'intervID': gb_str_shape('Intervention ID'),
                                                                                'delivChanID': gb_str_shape('Delivery Channel ID'),
                                                                                'value': gb_year_shape('Value')  # This is a percentage, so it is a float.
                                                                            },
                                                                            has_default=False
                                                                        ),
        icmvt.IC_MIN_PER_AVG_CASE_STATIC_STAFF_TAG_V1             :  gb_shape('IC Min per average case static staff', '1D dict',
                                                                            innerShape={
                                                                                'intervID': gb_str_shape('Intervention ID'),
                                                                                'delivChanID': gb_str_shape('Delivery Channel ID'),
                                                                                'defaultStaffTypeMstID': gb_int_shape('Default Staff Master ID'),
                                                                                'value': gb_year_shape('Value')  # This is a percentage, so it is a float.
                                                                            },
                                                                            has_default=False
                                                                        ),
        # icmvt.IC_MIN_PER_AVG_CASE_DYNAMIC_STAFF_TAG_V1            : value_list,
        icmvt.IC_FAM_PLAN_GOAL_TAG_V1                             : gb_int_shape('IC Family Planning Goal'),
        icmvt.IC_PERC_UNMET_NEED_COVERED_TAG_V1                   : gb_year_shape('Percentage Unmet Need Covered'), # This is a percentage, so it is a float.

        # Outputs / results

        icmvt.IC_RES_NUM_SERVICES_BY_DELIV_CHAN_TAG_V1           : gb_shape('IC Number of Services by Delivery Channel', '3D float',
                                                                            dim=[ic_interv_dim, ic_deliv_channel_dim, gb_year_dim],
                                                                            indices=[ic_interv_indices, ic_deliv_channel_indices, gb_year_indices]),
        icmvt.IC_RES_TARG_POPS_TAG_V1                            : gb_shape('IC Target Populations', '2D float', dims=[ic_interv_dim, gb_year_dim], indices=[ic_interv_indices, gb_year_indices]),
        icmvt.IC_RES_PEOPLE_NEEDING_SERVICES_TAG_V1              : gb_shape('IC Target Populations', '2D float', dims=[ic_interv_dim, gb_year_dim], indices=[ic_interv_indices, gb_year_indices]),
        icmvt.IC_RES_NUM_SERVICES_TAG_V1                         : gb_shape('IC Target Populations', '2D float', dims=[ic_interv_dim, gb_year_dim], indices=[ic_interv_indices, gb_year_indices]),
        icmvt.IC_RES_PERC_DELIV_THROUGH_CHAN_BY_YEAR_TAG_V1      : gb_shape('IC Number of Services by Delivery Channel', '3D float',
                                                                            dim=[ic_interv_dim, ic_deliv_channel_dim, gb_year_dim],
                                                                            indices=[ic_interv_indices, ic_deliv_channel_indices, gb_year_indices]),
        icmvt.IC_RES_DRUG_COSTS_TAG_V1                           : gb_shape('IC Labor Costs DSC', '3D float',
                                                                          dim=[ic_interv_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                                          indices=[ic_interv_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
        icmvt.IC_RES_DRUG_COSTS_BY_YEAR_ONLY_TAG_V1              : gb_year_shape('IC Drug Costs by Year Only'),
        icmvt.IC_RES_SUPPLY_COSTS_TAG_V1                         : gb_shape('IC Labor Costs DSC', '3D float',
                                                                          dim=[ic_interv_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                                          indices=[ic_interv_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
        icmvt.IC_RES_SUPPLY_COSTS_BY_YEAR_ONLY_TAG_V1            : gb_year_shape('IC Supply Costs by Year Only'),
        icmvt.IC_RES_STATIC_STAFF_TIME_REQ_BY_DELIV_CHAN_TAG_V1  : gb_shape('IC Static Staff Time Required by Delivery Channel', '3D float',
                                                                          dim=[ic_staff_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                                            indices=[ic_staff_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
        icmvt.IC_RES_STATIC_STAFF_TIME_REQ_BY_INTERV_TAG_V1      : gb_shape('IC Static Staff Time Required by Intervention', '2D float',
                                                                          dim=[ic_staff_dim, ic_interv_dim, gb_year_dim],
                                                                            indices=[ic_staff_indices, ic_interv_indices, gb_year_indices]),
        icmvt.IC_RES_STATIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1   : gb_shape('IC Static Staff Time Required by Program Area', '2D float',
                                                                          dim=[ic_staff_dim, ic_prog_area_dim, gb_year_dim],  
                                                                            indices=[ic_staff_indices, ic_prog_area_indices, gb_year_indices]),
        icmvt.IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_DELIV_CHAN_TAG_V1 : gb_shape('IC Dynamic Staff Time Required by Delivery Channel', '3D float',
                                                                            dim=[ic_staff_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                                            indices=[ic_staff_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
        icmvt.IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_INTERV_TAG_V1     : gb_shape('IC Dynamic Staff Time Required by Intervention', '3D float',
                                                                            dim=[ic_staff_dim, ic_interv_dim, gb_year_dim],
                                                                            indices=[ic_staff_indices, ic_interv_indices, gb_year_indices]),
        icmvt.IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1  : gb_shape('IC Dynamic Staff Time Required by Program Area', '3D float',
                                                                            dim=[ic_staff_dim, ic_prog_area_dim, gb_year_dim],
                                                                            indices=[ic_staff_indices, ic_prog_area_indices, gb_year_indices]),
        
        
        # These two are too large and need to be converted to records.
        icmvt.IC_RES_DRUGS_REQUIRED_BY_DELIV_CHAN_TAG            : gb_shape('IC Drugs Required by Delivery Channel', '3D float',
                                                                            dim=[ic_drug_supply_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                                            indices=[ic_drug_supply_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
                                                                            
        icmvt.IC_RES_SUPPLIES_REQUIRED_BY_DELIV_CHAN_TAG         : gb_shape('IC Supplies Required by Delivery Channel', '3D float',
                                                                            dim=[ic_drug_supply_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                                            indices=[ic_drug_supply_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
       
        icmvt.IC_RES_DRUGS_REQUIRED_BY_PROG_AREA_TAG            : gb_shape('IC Drugs Required by Program Area', '3D float',
                                                                           dim=[ic_drug_supply_dim, ic_prog_area_dim, gb_year_dim],
                                                                           indices=[ic_drug_supply_indices, ic_prog_area_indices, gb_year_indices]),
                                                                            
        
        icmvt.IC_RES_SUPPLIES_REQUIRED_BY_PROG_AREA_TAG         : gb_shape('IC Supplies Required by Program Area', '3D float',
                                                                           dim=[ic_drug_supply_dim, ic_prog_area_dim, gb_year_dim],
                                                                           indices=[ic_drug_supply_indices, ic_prog_area_indices, gb_year_indices]),
        
       
        icmvt.IC_RES_OUTPAT_VISITS_INPAT_DAYS_TAG_V1             : gb_shape('IC Outpatient Visits Inpatient Days', '4D float',
                                                                            dim=[ic_interv_dim, ic_visit_type_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                                            indices=[ic_interv_indices, ic_visit_type_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
		
		# Comprehensive vs specific plan (CVS)
        icmvt.IC_OUTPAT_INPAT_COSTS_CVS_TAG_V1 : gb_shape('IC Outpatient Inpatient Costs CVS', '2D float',
                                                           dims=[ic_visit_type_dim, ic_costed_deliv_chan_dim],
                                                           indices=[ic_visit_type_indices, ic_costed_deliv_chan_ids]),
        icmvt.IC_RES_LABOR_COSTS_DSC_TAG_V1 : gb_shape('IC Labor Costs DSC', '3D float',
                                                        dim=[ic_interv_dim, ic_costed_deliv_chan_dim, gb_year_dim],
                                                        indices=[ic_interv_indices, ic_costed_deliv_chan_ids, gb_year_indices]),
                                                                          
}


# Spectrum modules

# IHT modules
from SpectrumCommon.Const.IH.IHModVarFields import *
from SpectrumCommon.Const.PC.PCModVarFields import *
from SpectrumCommon.Const.IC.ICModVarFields import *
from SpectrumCommon.Const.IS.ISModVarFields import *
from SpectrumCommon.Const.HW.HWModVarFields import *
from SpectrumCommon.Const.LG.LGModVarFields import *
from SpectrumCommon.Const.BG.BGModVarFields import *

#############################    IH    ##########################

TR_P_PROG_AREA_ID = IH_F_PROG_AREA_ID
TR_P_SUBGROUP_ID = IH_F_SUBGROUP_ID
TR_P_DELIV_CHAN_ID = IH_F_DELIV_CHAN_ID
TR_P_HEALTH_SYS_ADMIN_UNIT_ID = IH_F_HEALTH_SYS_ADMIN_UNIT_ID

#############################    PC    ##########################

TR_P_COST_CAT_ID = PC_F_COST_CAT_ID
TR_P_COST_INPUT_ID = PC_F_COST_INPUT_ID
TR_P_AREA_ID = PC_F_AREA_ID
TR_P_AREA_TYPE_ID = PC_F_AREA_TYPE_ID
TR_P_COSTING_SECT_ID = PC_F_COSTING_SECT_ID
TR_P_COSTING_SUBSECT_ID = PC_F_COSTING_SUBSECT_ID
TR_P_ACT_ID = PC_F_ACT_ID

#############################    IC    ##########################

TR_P_INTERV_ID = IC_F_INTERV_ID
TR_P_DRUG_SUPPLY_TYPE_MST_ID = IC_F_DRUG_SUPPLY_TYPE_MST_ID
TR_P_DRUG_SUPPLY_ID = IC_F_DRUG_SUPPLY_ID
TR_P_TREAT_INPUT_ID = IC_F_TREAT_INPUT_ID
TR_P_TREAT_INPUT_GROUP_ID = IC_F_TREAT_INPUT_GROUP_ID
TR_P_APPLIC_PROG_AREA_IDS = IC_F_APPLIC_PROG_AREA_IDS
TR_P_CUSTOM_STAFF_TYPE = IC_F_CUSTOM_STAFF_TYPE

#############################    IS    ##########################

TR_P_FACILITY_TYPE_ID = IS_F_FACILITY_TYPE_ID
TR_P_FACILITY_MED_EQUIP_GROUP_ID = IS_F_FACILITY_MED_EQUIP_GROUP_ID
TR_P_FACILITY_EQUIP_ID = IS_F_FACILITY_EQUIP_ID
TR_P_ICT_EQUIP_ID = IS_F_ICT_EQUIP_ID
TR_P_VEHICLE_TYPE_ID = IS_F_VEHICLE_TYPE_ID
TR_P_FACILITY_SUPP_TYPE_MST_ID = IS_FACIL_SUPP_TYPE_MST_ID
TR_P_DEF_FACILITY_TYPE_MAP_MST_ID = IS_FACIL_DEFAULT_MAP_MST_ID

#############################    HW    ##########################

TR_P_STAFF_TYPE_ID = HW_F_STAFF_TYPE_ID
TR_P_ATTR_TYPE_ID = HW_F_ATTR_TYPE_ID
TR_P_RETENT_INCENT_ID = HW_F_RETENT_INCENT_ID
TR_P_STAFF_DIVISION_MST_ID = HW_STAFF_DIVISION_MST_ID
TR_P_STATIC_STAFF_TYPE_MAP_MST_ID = HW_STAFF_STATIC_MAP_MST_ID

#############################   BG   ##############################

TR_P_BUDGET_ID = BG_F_BUDGET_ID
TR_P_MOD_ID = BG_F_MOD_ID
TR_P_BUDGET_CAT_ID = BG_F_BUDGET_CAT_ID
TR_P_FUND_FW_ID = BG_F_FUND_FW_ID
TR_P_FUND_SOURCE_ID = BG_F_FUND_SOURCE_ID
TR_P_IND_TYPE_ID = BG_F_IND_TYPE_ID
TR_P_IND_ID = BG_F_IND_ID
TR_P_IS_AGGREGATE = BG_F_IS_AGGREGATE
TR_P_TARGET_ID = BG_QUICK_MAP_TARGET_ID
TR_P_ON = BG_QUICK_MAP_ON
TR_P_STAFF_LEVEL_ID = BG_F_STAFF_LEVEL_ID_HW
TR_P_COMPENS_TYPE_ID = BG_F_COMPENS_TYPE_ID_HW




TB_UseOldFailsafeTag   = '<TB_UseOldFailsafe_V1>'
TB_UseOldIPTag         = '<TB_UseIPModel_V1>'
TB_BaseYearTag         = '<TB_BaseYear_V2>'
TB_FirstYearIdxTag     = '<TB_FirstYearIdx_V1>' # deprecated
TB_ScreeningAlgMapTag  = '<TB_ScreeningAlgMap_V1>'
# TB_ImpactModelTag      = '<TB_ImpactModelSelected_V1>'

TB_HistNotifStartYearTag = '<TB_HistNotifStartYear_V1>'
TB_HistNotifTag = '<TB_HistNotif_V1>'
TB_HistNotiDistributionTag = '<TB_HistNotiDistribution_V1>'

# Household demographic info    
TB_HHSizeTag         = '<TB_HHSize_V1>'# Average household size (#)
TB_HHPropU5Tag       = '<TB_HHPropU5_V1>'#  Proportion of household that is u5 (# )
TB_HHProp5_14Tag     = '<TB_HHProp5_14_V1>'# Proportion of household that is 5-14 yrs 
TB_HHCasesinIndexTag = '<TB_HHCasesinIndex_V1>'# Average number of TB cases in household with at least one case (#)

# TB infection
TB_HHPropU5LTBITag = '<TB_HHPropU5LTBI_V1>' # Proportion of u5s with TB infection in household with TB infection (# )
TB_HHPropO5LTBITag = '<TB_HHPropO5LTBI_V1>' # Proportion of other household members with TB infection in household with active case (# )

# TB disease
TB_HHPropU5TBTag  = '<TB_HHPropU5TB_V1>'  # Proportion of�u5s with TB disease in household with TB disease (# )
TB_HHPropU5PTBTag = '<TB_HHPropU5PTB_V1>' # Proportion with pulmonary TB, PTB (# )
TB_HHPropO5TBTag  = '<TB_HHPropO5TB_V1>'  # Proportion of other household members with TB disease in household with TB disease (# )
TB_HHPropO5PTBTag = '<TB_HHPropO5PTB_V1>'# Proportion with pulmonary TB, PTB (# )
TB_HHNumNewBactPosTag = '<TB_HHNumNewBactPos_V1>'# Number of new bacteriologically positive TB cases (adult)
TB_HHNumNewBactNegTag = '<TB_HHNumNewBactNeg_V1>'# Number of new bacteriologically negative TB cases (adult)

TB_HHPIndexCasesScreenedTag     = '<TB_HHIndexCasesScreened_V1>'     # Proportion of households of index cases screened/evaluated
TB_HHProportionTag       = '<TB_HHProportionScreen_V1>'
TB_HHPropBACnScreenedTag = '<TB_HHPropBACnScreened_V1>' # Proportion of contacts of new bacteriologically negative cases screened/evaluated

TB_HHContacts = '<TB_HHContacts_V1>'   
TB_HHScreeningAlgTag           = '<TB_HHScreeningAlg_V1>'
TB_HHScreeningAlgIDTag         = '<TB_HHScreeningAlgID_V1>'
TB_HHScreenSensitivityTag      = '<TB_HHScreenSensitivity_V1>' 
TB_HHScreenSpecificityTag      = '<TB_HHScreenSpecificity_V1>' 
TB_HHScreenAlgOptionsTag       = '<TB_HHScreenAlgOptions_V2>'
TB_HHScreenAlgSelectedTag      = '<TB_HHScreenAlgSelected_V1>'   
TB_ScreenCustomTag             = '<TB_ScreenCustom_V1>' 
TB_DiagnosticCustomTag         = '<TB_DiagnosticCustom_V1>' 
TB_HHDiagSensitivityTag        = '<TB_HHDiagSensitivity_V1>' 
TB_HHDiagSpecificityTag        = '<TB_HHDiagSpecificity_V1>'            
TB_HHNumScreenedTag            = '<TB_HHNumScreened_V1>'        
TB_HHNumReferredTag            = '<TB_HHNumReferred_V1>'                  
TB_HHNumScreenedToReferTag     = '<TB_HHNumScreenedToRefer_V1>'           
TB_HHPercTrueCasesTag          = '<TB_HHPercTrueCases_V1>'          
TB_HHDiagCaseValuesTag         = '<TB_HHDiagCaseValues_V1>'                      
TB_HHDiagCaseIDsTag            = '<TB_HHDiagCaseIDs_V1>'                    
TB_HHDiagCaseMethodsTag        = '<TB_HHDiagCaseMethodIDs_V1>'  
TB_HHDiagCaseSensitivityTag    = '<TB_HHDiagCaseSensitivity_V1>'
TB_HHDiagCaseSpecificityTag    = '<TB_HHDiagCaseSpecificity_V1>'
TB_HHNumDiagnosedPTBTag        = '<TB_HHNumDiagnosedPTB_V1>'              
TB_HHNumConfirmedPTBTag        = '<TB_HHNumConfirmedPTB_V1>'           
TB_HHPropLinkedTreatTag        = '<TB_HHPropLinkedTreatment_V1>'    
TB_HHNumInitTreatmentTag       = '<TB_HHNumInitTreatment_V1>'  
TB_HHPrvntInActiveEligibleTag  = '<TB_HHPrvntInActiveEligible_V1>'  
TB_HHPrvntReceivePTPrevTag    = '<TB_HHPrvntReceivevPTPrev_V1>'  
TB_HHPropPrvntPresumpTag       = '<TB_HHPrvntGivenPT_V1>'  
TB_HHPropPrvntTestedTag        = '<TB_HHPrvntTestedLTBI_V1>'  
TB_HHPrvntMethodsTag           = '<TB_HHPrvntMethods_V2>'  
TB_HHPrvntMethodIDsTag         = '<TB_HHPrvntMethodIDs_V2>'  
TB_HHPropEligiblePrvntLinkTag  = '<TB_HHPrvntEligibleCases_V1>'   
TB_HHNumberOnTPTTag            = '<TB_HHNumberOnTPT_V1>'
TB_HHTPTCompleteRateTag        = '<TB_HHTPTCompleteRate_V1>'
TB_HHNumberCompleteTPTTag      = '<TB_HHNumberCompleteTPT_V1>' 
TB_HHScreeningOutputTag        = '<TB_HHScreeningOutput_V1>' 
TB_HHDiagnosticOutputTag       = '<TB_HHDiagnosticOutput_V1>' 
TB_HHTargetPopsTag             = '<TB_HHTargetPops_V1>'

TB_ARTCohortSizesTag            = '<TB_ARTCohortSizes_V1>'
TB_ARTCharacteristicsTag        = '<TB_ARTCharacteristics_V1>'

TB_ARTProportionTag             = '<TB_ARTProportionScreen_V1>'
TB_ARTScreeningAlgTag           = '<TB_ARTScreeningAlg_V1>'
TB_ARTScreeningAlgIDTag         = '<TB_ARTScreeningAlgID_V1>'
TB_ARTScreenSensitivityTag      = '<TB_ARTScreenSensitivity_V1>' 
TB_ARTScreenSpecificityTag      = '<TB_ARTScreenSpecificity_V1>'  
TB_ARTScreenAlgOptionsTag       = '<TB_ARTScreenAlgOptions_V2>'
TB_ARTScreenAlgSelectedTag      = '<TB_ARTScreenAlgSelected_V1>'
TB_ARTDiagSensitivityTag        = '<TB_ARTDiagSensitivity_V1>' 
TB_ARTDiagSpecificityTag        = '<TB_ARTDiagSpecificity_V1>'                
TB_ARTNumScreenedTag            = '<TB_ARTNumScreened_V1>'        
TB_ARTNumReferredTag            = '<TB_ARTNumReferred_V1>'                  
TB_ARTNumScreenedToReferTag     = '<TB_ARTNumScreenedToRefer_V1>'           
TB_ARTPercTrueCasesTag          = '<TB_ARTPercTrueCases_V1>'          
TB_ARTDiagCaseValuesTag         = '<TB_ARTDiagCaseValues_V2>'                      
TB_ARTDiagCaseIDsTag            = '<TB_ARTDiagCaseIDs_V2>'                    
TB_ARTDiagCaseMethodsTag        = '<TB_ARTDiagCaseMethodIDs_V2>'   
TB_ARTDiagCaseSensitivityTag    = '<TB_ARTDiagCaseSensitivity_V2>'
TB_ARTDiagCaseSpecificityTag    = '<TB_ARTDiagCaseSpecificity_V2>'            
TB_ARTNumDiagnosedPTBTag        = '<TB_ARTNumDiagnosedPTB_V1>'              
TB_ARTNumConfirmedPTBTag        = '<TB_ARTNumConfirmedPTB_V1>'           
TB_ARTPropLinkedTreatTag        = '<TB_ARTPropLinkedTreatment_V1>'    
TB_ARTNumInitTreatmentTag       = '<TB_ARTNumInitTreatment_V1>'  
TB_ARTPrvntInActiveEligibleTag  = '<TB_ARTPrvntInActiveEligible_V1>'  
TB_ARTPrvntReceivePTPrevTag    = '<TB_ARTPrvntReceivevPTPrev_V1>'  
TB_ARTPropPrvntPresumpTag       = '<TB_ARTPrvntGivenPT_V1>'  
TB_ARTPropPrvntTestedTag        = '<TB_ARTPrvntTestedLTBI_V1>'  
TB_ARTPrvntMethodsTag           = '<TB_ARTPrvntMethods_V1>'  
TB_ARTPrvntMethodIDsTag         = '<TB_ARTPrvntMethodIDs_V1>'  
TB_ARTPropEligiblePrvntLinkTag  = '<TB_ARTPrvntEligibleCases_V1>' 
TB_ARTNumberOnTPTTag            = '<TB_ARTNumberOnTPT_V1>'
TB_ARTTPTCompleteRateTag        = '<TB_ARTTPTCompleteRate_V1>'
TB_ARTNumberCompleteTPTTag      = '<TB_ARTNumberCompleteTPT_V1>' 
TB_ARTScreeningOutputTag        = '<TB_ARTScreeningOutput_V1>' 
TB_ARTDiagnosticOutputTag       = '<TB_ARTDiagnosticOutput_V1>' 
TB_ARTTargetPopsTag             = '<TB_ARTargetPops_V1>'


TB_HRGDemographicsTag           = '<TB_HRGDemographics_V1>'

TB_HRGProportionTag             = '<TB_HRGProportionScreen_V1>'
TB_HRGScreeningAlgTag           = '<TB_HRGScreeningAlg_V1>'
TB_HRGScreeningAlgIDTag         = '<TB_HRGScreeningAlgID_V1>'
TB_HRGScreenSensitivityTag      = '<TB_HRGScreenSensitivity_V1>' 
TB_HRGScreenSpecificityTag      = '<TB_HRGScreenSpecificity_V1>' 
TB_HRGScreenAlgOptionsTag       = '<TB_HRGScreenAlgOptions_V2>'
TB_HRGScreenAlgSelectedTag      = '<TB_HRGScreenAlgSelected_V1>'
TB_HRGDiagSensitivityTag        = '<TB_HRGDiagSensitivity_V1>' 
TB_HRGDiagSpecificityTag        = '<TB_HRGDiagSpecificity_V1>'                  
TB_HRGNumScreenedTag            = '<TB_HRGNumScreened_V1>'        
TB_HRGNumReferredTag            = '<TB_HRGNumReferred_V1>'                  
TB_HRGNumScreenedToReferTag     = '<TB_HRGNumScreenedToRefer_V1>'           
TB_HRGPercTrueCasesTag          = '<TB_HRGPercTrueCases_V1>'          
TB_HRGDiagCaseValuesTag         = '<TB_HRGDiagCaseValues_V1>'                      
TB_HRGDiagCaseIDsTag            = '<TB_HRGDiagCaseIDs_V1>'                    
TB_HRGDiagCaseMethodsTag        = '<TB_HRGDiagCaseMethodIDs_V1>' 
TB_HRGDiagCaseSensitivityTag    = '<TB_HRGDiagCaseSensitivity_V1>'
TB_HRGDiagCaseSpecificityTag    = '<TB_HRGDiagCaseSpecificity_V1>'                  
TB_HRGNumDiagnosedPTBTag        = '<TB_HRGNumDiagnosedPTB_V1>'              
TB_HRGNumConfirmedPTBTag        = '<TB_HRGNumConfirmedPTB_V1>'           
TB_HRGPropLinkedTreatTag        = '<TB_HRGPropLinkedTreatment_V1>'    
TB_HRGNumInitTreatmentTag       = '<TB_HRGNumInitTreatment_V1>'  
TB_HRGPrvntInActiveEligibleTag  = '<TB_HRGPrvntInActiveEligible_V1>'  
TB_HRGPrvntReceivePTPrevTag    = '<TB_HRGPrvntReceivevPTPrev_V1>'  
TB_HRGPropPrvntPresumpTag       = '<TB_HRGPrvntGivenPT_V1>'  
TB_HRGPropPrvntTestedTag        = '<TB_HRGPrvntTestedLTBI_V1>'  
TB_HRGPrvntMethodsTag           = '<TB_HRGPrvntMethods_V1>'  
TB_HRGPrvntMethodIDsTag         = '<TB_HRGPrvntMethodIDs_V1>'  
TB_HRGPropEligiblePrvntLinkTag  = '<TB_HRGPrvntEligibleCases_V1>' 
TB_HRGNumberOnTPTTag            = '<TB_HRGNumberOnTPT_V1>'
TB_HRGTPTCompleteRateTag        = '<TB_HRGTPTCompleteRate_V1>'
TB_HRGNumberCompleteTPTTag      = '<TB_HRGNumberCompleteTPT_V1>' 
TB_HRGScreeningOutputTag        = '<TB_HRGScreeningOutput_V1>' 
TB_HRGDiagnosticOutputTag       = '<TB_HRGDiagnosticOutput_V1>' 
TB_HRGTargetPopsTag             = '<TB_HRGTargetPops_V1>'

# Prevalence of active TB    
# Pulmonary TB 
# TB_PIHIVn014_PTBTag  = '<TB_PIHIVn014_PTB_V1>' 
# TB_PIHIVn15p_PTBTag  = '<TB_PIHIVn15p_PTB_V1>' 
# TB_PIHIVn09_PTBTag   = '<TB_PIHIVn09_PTB_V1>' 
# TB_PIHIVn1014_PTBTag = '<TB_PIHIVn1014_PTB_V1>' 
# TB_PIHIVn15p_PTBTag  = '<TB_PIHIVn15p_PTB_V1>' 
# # Extra-Pulmonary TB
# TB_PIHIVp014_ETBTag  = '<TB_PIHIVp014_ETB_V1>' 
# TB_PIHIVp15p_ETBTag  = '<TB_PIHIVp15p_ETB_V1>' 
# TB_PIHIVp09_ETBTag   = '<TB_PIHIVp09_ETB_V1>' 
# TB_PIHIVp1014_ETBTag = '<TB_PIHIVp1014_ETB_V1>' 
# TB_PIHIVp15p_ETBTag  = '<TB_PIHIVp15p_ETB_V1>' 



        
# TB_PropScreenedTag  = '<TB_PropScreened_V1>'

# Screening algorithm used for screening
# Sensitivity for PTB
# Specificity for PTB
TB_PatientScreenSensTag = '<TB_PatientScreenSens_V1>' 
TB_PatientScreenSpecTag = '<TB_PatientScreenSpec_V1>' 
TB_DiagSensTag   = '<TB_DiagSens_V1>' 
TB_DiagSpecTag   = '<TB_DiagSpec_V1>' 


# TB_PropDiagLinkedTxTag       = '<TB_PropDiagLinkedTx_V1>'      
# TB_PropTPTPresumptiveTxTag   = '<TB_PropTPTPresumptiveTx_V1>'# Proportion given PT presumptively
# TB_PropEligibleTPTtestedTag  = '<TB_PropEligibleTPTtested_V1>'# Proportion of remaining household contacts tested for LTBI
# TB_PropEligibleLinkedTPTTag  = '<TB_PropEligibleLinkedTPT_V1>'# Proportion of presumed or confirmed TB preventive treatment-eligible cases linked to TB preventive treatment

# TB_ART09_PropNotSeriouslyIllTag    = '<TB_ART09_PropNotSeriouslyIll_V1>'
# TB_ART09_PropSeriouslyIllTag       = '<TB_ART09_PropSeriouslyIll_V1>'
# TB_ART1014_PropNotSeriouslyIllTag  = '<TB_ART1014_PropNotSeriouslyIll_V1>'
# TB_ART1014_PropSeriouslyIllTag     = '<TB_ART1014_PropSeriouslyIll_V1>'
# TB_ART15p_PropNotSeriouslyIllTag   = '<TB_ART15p_PropNotSeriouslyIll_V1>' # Without serious illness (# )
# TB_ART15p_PropSeriouslyIllTag      = '<TB_ART15p_PropSeriouslyIll_V1>' # With serious illness (# )  
# # ART, TB infection and disease characteristics
# TB_ART09_PropLTBITag   = '<TB_ART09_PropLTBI_V1>' # Proportion of�ART patients 0 to 9 years with TB infection (# )
# TB_ART09_PropTBTag     = '<TB_ART09_PropTB_V1>' # Proportion of�ART patients 0 to 9 years with TB disease (# )
# TB_ART09_PropPTBTag    = '<TB_RT09_PropPTB_V1>' # Proportion with pulmonary TB, PTB (# )
# TB_ART1014_PropLTBITag = '<TB_ART1014_PropLTBI_V1>' # Proportion of�ART patients 10 to 14 years with TB infection (# )        
# TB_ART1014_PropTBTag   = '<TB_ART1014_PropTB_V1>' # Proportion of�ART patients 10 to 14 years with TB disease (# )
# TB_ART1014_PropPTBTag  = '<TB_ART1014_PropPTB_V1>' # Proportion with pulmonary TB, PTB (# )
# TB_ART15p_PropLTBITag  = '<TB_ART15p_PropLTBI_V1>' # Proportion of�ART patients 15 years and older with TB infection (# )
# TB_ART15p_PropTBTag    = '<TB_ART15p_PropTB_V1>' # Proportion of�ART patients 15 years and older with TB disease (# )    
# TB_ART15p_PropPTBTag   = '<TB_ART15p_PropPTB_V1>' # Proportion with pulmonary TB, PTB (# )


        
# TB_PropPop15pHRGTag = '<TB_PropPop15pHRG_V1>' # Proportion of population 15+ in HRG    
# TB_HRG_PropLTBITag  = '<TB_HRG_PropLTBI_V1>' # Proportion of�ART patients 15 years and older with TB infection (# )
# TB_HRG_PropTBTag    = '<TB_HRG_PropTB_V1>' # Proportion of�ART patients 15 years and older with TB disease (# )
# TB_HRG_PropPTBTag   = '<TB_HRG_PropPTB_V1>' # Proportion with pulmonary TB, PTB (# )

# From the wireframes
TB_PulmonaryAgeTag                 = '<TB_PulmonaryAges_V1>'   
TB_PrevalenceTag                   = '<TB_Prevalence_V1>'                          

TB_PatientScreenSensitivityTag     = '<TB_PatientScreenSensitivity_V1>'                  
TB_PatientScreenSpecificityTag     = '<TB_PatientScreenSpecificity_V1>' 
TB_PatientScreenAlgOptionsTag      = '<TB_PatientScreenAlgOptions_V2>'
TB_PatientScreenAlgSelectedTag     = '<TB_PatientScreenAlgSelected_V1>'
                
TB_NumClinicallyAssessedTag        = '<TB_NumClinicallyAssessed_V1>'        
TB_NumReferredTag                  = '<TB_NumReferred_V1>'                  
TB_NumScreenedToReferTag           = '<TB_NumScreenedToRefer_V1>'           
TB_PercTrueCasesTag                = '<TB_PercTrueCases_V1>' 

TB_PatientDiagCaseValuesTag         = '<TB_PatientDiagCaseValues_V1>'                      
TB_PatientDiagCaseIDsTag            = '<TB_PatientDiagCaseIDs_V1>'                      
TB_PatientDiagCaseMethodsTag      = '<TB_PatientDiagCaseMethodIDs_V1>'   
TB_PatientDiagCaseSensitivityTag    = '<TB_PatientDiagCaseSensitivity_V1>'   
TB_PatientDiagCaseSpecificityTag    = '<TB_PatientDiagCaseSpecificity_V1>'  

TB_PatientDiagSensitivityTag       = '<TB_PatientDiagSensitivity_V1>'                  
TB_PatientDiagSpecificityTag       = '<TB_PatientDiagSpecificity_V1>' 

TB_NumDiagnosedPTBTag              = '<TB_NumDiagnosedPTB_V1>'              
TB_NumConfirmedPTBTag              = '<TB_NumConfirmedPTB_V1>'      
TB_NumDiagnosedTBTag               = '<TB_NumDiagnosedTB_V1>' 
TB_NumInitTreatmentTag             = '<TB_NumInitTreatment_V1>'  

TB_PatientScreeningOutputTag       = '<TB_PatientScreeningOutput_V1>'
TB_PatientDiagnosticOutputTag      = '<TB_PatientDiagnosticOutput_V1>'
TB_PatientTargetPopTag             = '<TB_PatientTargetPop_V1>'

TB_HisNotificationTag  = '<TB_HistoricalNotification_V1>'
TB_HisIncidenceTag     = '<TB_HistoricalIncidence_V1>'
TB_HisMortalityTag     = '<TB_HistoricalMortality_V1>'
TB_RetNrelPercTag      = '<TB_ret_nrel_perc_V1>'
TB_TargNotificationTag = '<TB_TargetNotification_V1>'
TB_ResNotificationTag  = '<TB_ResNotification_V1>'
TB_ResNotificationV2Tag  = '<TB_ResNotification_V2>'

TB_ResPrevalenceTag    = '<TB_ResPrevalence_V1>'
TB_ResPrevalencePctTag = '<TB_ResPrevalencePct_V1>'
TB_ResHIVTBCoInfTag    = '<TB_ResHIVTBCoInfTag_V1>'

TB_ResIncidenceTag     = '<TB_ResIncidence_V1>'
TB_ResMortalityTag     = '<TB_ResMortality_V1>'

TB_UserEnteredIncidenceTag     = '<TB_UserEnteredIncidence_V1>'
TB_UserEnteredMortalityTag     = '<TB_UserEnteredMortality_V1>'

TB_CounterfactualScenarioTag  = '<TB_CounterfactualScenario_V1>'
TB_CurrentScenarioTag         = '<TB_CurrentScenario_V1>'

TB_CounterfactualNotifTag = '<TB_CounterfactualNotification_V1>'

TB_FortIncidenceTag    = '<TB_FortIncidence_V1>'
TB_FortNotificationTag = '<TB_FortNotification_V1>'
TB_FortPrevalenceTag   = '<TB_FortPrevalence_V1>'
TB_FortMortalityTag    = '<TB_FortMortality_V1>'


TB_NotificationByCaseTypeTag = '<TB_NotificationByCaseType_V2>'
TB_PercNotificationTag = '<TB_PercentNotification_V2>'
TB_Baseline_HRi_allTag = '<TB_Baseline_HRi_all_V1>'
TB_Scaleup_HRi_allTag = '<TB_Scaleup_HRi_all_V1>'

TB_SubnationalRatioTag    = '<TB_SubnationalRatio_V1>'

TB_AlternativeProjBurdenTag = '<TB_AlternativeProjBurden_V1>'



TB_ResistProfileTag          = '<TB_ResistProfile_V1>'
TB_ResistPropNonSevereTag    = '<TB_ResistPropNonSevere_V1>'
TB_ResistPropLungSurgeryTag  = '<TB_ResistPropLungSurgery_V1>'
TB_ResistMeningitisTag       = '<TB_ResistMeningitis_V1>'
TB_ResistPropTreatExhaustTag = '<TB_ResistPropTreatExhaust_V1>'
TB_PatientTestNumPTB         = '<TB_PatientTestNumPTB_V1>'
TB_PatientTestNumXDR         = '<TB_PatientTestNumXDR_V1>'
TB_PatientTestNumEPTB        = '<TB_PatientTestNumETB_V1>'
TB_CoverageTag               = '<TB_Coverage_V1>'
TB_ResistNumToBeTestedTag    = '<TB_ResistNumToBeTested_V1>'
TB_ResistNumTestedTag        = '<TB_ResistNumTested_V1>'
TB_ResistNumConfirmedTag     = '<TB_ResistNumConfirmed_V1>'
TB_ResistNumNotConfirmedTag  = '<TB_ResistNumNotConfirmed_V1>'
TB_PropPreventionTag         = '<TB_PropPrevention_V1>'
TB_BCGImmunizationTag        = '<TB_BCGImmunization_V1>'
TB_InfantVaccineTag          = '<TB_InfantVaccine_V1>'
TB_AdolAdultVaccineTag       = '<TB_AdolAdultVaccine_V1>'
TB_PalliativeCareTag         = '<TB_PalliativeCare_V1>'

TB_BCGTargetPopTag          = '<TB_BCGTargetPop_V1>'
TB_InfantVaccTargPopTag     = '<TB_InfantVaccineTargPop_V1>'
TB_AdultVaccTargPopTag      = '<TB_AdolAdultVaccineTargPop_V1>'
# TB_PalliativeCareTargPopTag = '<TB_PalliativeCareTargPop_V1>'
TB_PalliativeCareTargPopTag = '<TB_PalliativeCareTargPop_V2>'


TB_PreventionTargetTag       = '<TB_PreventionTargets_V1>'


TB_RR_DST_PTBTag     = '<TB_RR_DST_PTB_V1>'
TB_RR_DST_ETBTag     = '<TB_RR_DST_ETB_V1>'
TB_INH_DST_PTBTag    = '<TB_INH_DST_PTB_V1>'
TB_INH_DST_ETBTag    = '<TB_INH_DST_ETB_V1>'
TB_FQ_DST_PTBTag     = '<TB_FQ_DST_PTB_V1>'
TB_FQ_DST_ETBTag     = '<TB_FQ_DST_ETB_V1>'
TB_BDQ_DST_PTBTag    = '<TB_BDQ_DST_PTB_V1>'
TB_BDQ_DST_ETBTag    = '<TB_BDQ_DST_ETB_V1>'
TB_BDQ_FQ_DST_PTBTag = '<TB_BDQ_FQ_DST_PTB_V1>'
TB_BDQ_FQ_DST_ETBTag = '<TB_BDQ_FQ_DST_ETB_V1>'
TB_LZD_RR_DST_PTBTag = '<TB_LZD_RR_DST_PTB_V1>'
TB_LZD_RR_DST_ETBTag = '<TB_LZD_RR_DST_ETB_V1>'
TB_LZD_FQ_DST_PTBTag = '<TB_LZD_FQ_DST_PTB_V1>'
TB_LZD_FQ_DST_ETBTag = '<TB_LZD_FQ_DST_ETB_V1>'
TB_PZA_DSTTag        = '<TB_PZA_DST_V1>'
TB_ETO_DSTTag        = '<TB_ETO_DST_V1>'
TB_EMC_DSTTag        = '<TB_EMC_DST_V1>'

TB_BDQ_LZD_NumDx_U15Tag         = '<TB_BDQ_LZD_NumDx_U15_V1>'
TB_BDQ_LZD_NumDx_15PLusTag      = '<TB_BDQ_LZD_NumDx_15PLus_V1>'
TB_BDQ_LZD_NumDx_PTB_U15Tag     = '<TB_BDQ_LZD_NumDx_PTB_U15_V1>'
TB_BDQ_LZD_NumDx_PTB_15PLusTag  = '<TB_BDQ_LZD_NumDx_PTB_15PLus_V1>'
TB_BDQ_LZD_NumDx_ETB_U15Tag     = '<TB_BDQ_LZD_NumDx_ETB_U15_V1>'
TB_BDQ_LZD_NumDx_ETB_15PLusTag  = '<TB_BDQ_LZD_NumDx_ETB_15PLus_V1>'


TB_AmikacynRifampicinResistant     = '<TB_AmikacynRifampicinResistant_V1>'
TB_EthionamideRifampicinResistant  = '<TB_EthionamideRifampicinResistant_V1>'
TB_PyrazinamideRifampicinResistant = '<TB_PyrazinamideRifampicinResistant_V1>'


TB_Rif_INH_Sen_SevereTag     = '<TB_Rif_INH_Sen_Severe_V1>'
TB_Rif_INH_Sen_NotSevereTag  = '<TB_Rif_INH_Sen_NotSevere_V1>'
TB_Rif_INH_Sen_MeningitisTag = '<TB_Rif_INH_Sen_Meningitis_V1>'
TB_Rif_INH_ResTag            = '<TB_Rif_INH_Res_V1>'
TB_RR_FQ_SenTag              = '<TB_RR_FQ_Sen_V1>'
TB_Rif_FQ_Res_PlusTag        = '<TB_Rif_FQ_Res_V1>'
TB_BQD_LZD_Tag               = '<TB_BQD_LZD_V1>' # Number confirmed Pre-XDR and resistant to at least one of Bedaquiline, Linezolid, Levofloxacin, Moxifloxacin';

TB_TxFL_Tag                  = '<TB_TxFL_V1>'
TB_TxSL_Tag                  = '<TB_TxSL_V1>'
TB_TxPreXDR_Tag              = '<TB_TxPreXDR_V1>'

TB_LungSurgeryTag            = '<TB_LungSurgery_V1>'

TB_lPTB_ReferredDxTag        = '<TB_lPTB_ReferredDxT_V1>'

TB_HIVnClinicallyAssessedTag  = '<TB_HIVnClinicallyAssessed_V1>'
TB_HIVAnyReferredDxTag        = '<TB_HIVAnyReferredDx_V1>'

TB_DynProportionsTag   = '<TB_DynamicalModelProportions_V1>'
TB_DynRatesTag         = '<TB_DynamicalModelRates_V1>'
TB_DynPRMTag           = '<TB_DynamicalModelPRM_V1>'
TB_DynUsedFitParamTag  = '<TB_DynamicalModelUsedFitParam_V1>'
TB_DynUsedHIVIncdTag   = '<TB_DynamicalModelHIVIncd_V1>'
TB_DynRangesTag        = '<TB_DynamicalModelRange_V1>'
TB_DynXSTag            = '<TB_DynamicalModelXS_V1>'
TB_DynPopDataTag       = '<TB_DynamicalModelPopData_V1>'

TB_FortVsnTag = '<TB_FortVersion_V1>'
TB_DynamicalVsnTag = '<TB_DynamicalVersion_V1>'

TB_PatientCustStepMenu  = '<TB_PatientCustStepMenu_V1>'
TB_PatientCustStep1     = '<TB_PatientCustStep1_V1>'
TB_PatientCustStep2     = '<TB_PatientCustStep2_V1>'
TB_PatientCustParallel = '<TB_PatientCustParallelOpt_V1>'

TB_HHCustStepMenu  = '<TB_HHCustStepMenu_V1>'
TB_HHCustStep1     = '<TB_HHCustStep1_V1>'
TB_HHCustStep2     = '<TB_HHCustStep2_V1>'
TB_HHCustParallel  = '<TB_HHCustParallelOpt_V1>'

TB_ARTCustStepMenu  = '<TB_ARTCustStepMenu_V1>'
TB_ARTCustStep1     = '<TB_ARTCustStep1_V1>'
TB_ARTCustStep2     = '<TB_ARTCustStep2_V1>'
TB_ARTCustParallel  = '<TB_ARTCustParallelOpt_V1>'

TB_HRGCustStepMenu  = '<TB_HRGCustStepMenu_V1>'
TB_HRGCustStep1     = '<TB_HRGCustStep1_V1>'
TB_HRGCustStep2     = '<TB_HRGCustStep2_V1>'
TB_HRGCustParallel  = '<TB_HRGCustParallelOpt_V1>'
TB_DynamicalFirstYearTag = '<TB_DynamicalFirstYear_V1>'
TB_DynamicalFinalYearTag = '<TB_DynamicalFinalYear_V1>'

TB_DataSourceTag = '<TB_DataSource_V1>'

TB_FinalHistoricalYearTag = '<TB_FinalHistoricalYear_V1>'
TB_FortInputVersionTag = '<TB_FortInputVersion_V1>'
TB_FortInputTag = '<TB_FortInputTag_V1>'
TB_FortInputScaleupTag = '<TB_FortInputScaleupTag_V1>'
TB_FortOutputVersionTag = '<TB_FortOutputVersion_V1>'
TB_FortOutputTag = '<TB_FortOutputTag_V1>'

TB_ALL_TAGS = [
    TB_BaseYearTag,
    TB_FirstYearIdxTag,
    TB_ScreeningAlgMapTag,
    TB_HHSizeTag,
    TB_HHPropU5Tag,
    TB_HHProp5_14Tag,
    TB_HHCasesinIndexTag,
    TB_HHPropU5LTBITag,
    TB_HHPropO5LTBITag,
    TB_HHPropU5TBTag,
    TB_HHPropU5PTBTag,
    TB_HHPropO5TBTag,
    TB_HHPropO5PTBTag,
    TB_HHNumNewBactPosTag,
    TB_HHNumNewBactNegTag,
    TB_HHPIndexCasesScreenedTag,
    TB_HHProportionTag,
    TB_HHPropBACnScreenedTag,
    TB_HHContacts,
    TB_HHScreeningAlgTag,
    TB_HHScreeningAlgIDTag,
    TB_HHScreenSensitivityTag,
    TB_HHScreenSpecificityTag,
    TB_HHScreenAlgOptionsTag,
    TB_HHScreenAlgSelectedTag,
    TB_HHDiagSensitivityTag,
    TB_HHDiagSpecificityTag,
    TB_HHNumScreenedTag,
    TB_HHNumReferredTag,
    TB_HHNumScreenedToReferTag,
    TB_HHPercTrueCasesTag,
    TB_HHDiagCaseValuesTag,
    TB_HHDiagCaseIDsTag,
    TB_HHDiagCaseMethodsTag,
    TB_HHDiagCaseSensitivityTag,
    TB_HHDiagCaseSpecificityTag,
    TB_HHNumDiagnosedPTBTag,
    TB_HHNumConfirmedPTBTag,
    TB_HHPropLinkedTreatTag,
    TB_HHNumInitTreatmentTag,
    TB_HHPrvntInActiveEligibleTag,
    TB_HHPropPrvntPresumpTag,
    TB_HHPropPrvntTestedTag,
    TB_HHPrvntMethodsTag,
    TB_HHPrvntMethodIDsTag,
    TB_HHPropEligiblePrvntLinkTag,
    TB_HHScreeningOutputTag,
    TB_HHDiagnosticOutputTag,
    TB_HHTargetPopsTag,
    TB_ARTCohortSizesTag,
    TB_ARTCharacteristicsTag,
    TB_ARTProportionTag,
    TB_ARTScreeningAlgTag,
    TB_ARTScreeningAlgIDTag,
    TB_ARTScreenSensitivityTag,
    TB_ARTScreenSpecificityTag,
    TB_ARTScreenAlgOptionsTag,
    TB_ARTScreenAlgSelectedTag,
    TB_ARTDiagSensitivityTag,
    TB_ARTDiagSpecificityTag,
    TB_ARTNumScreenedTag,
    TB_ARTNumReferredTag,
    TB_ARTNumScreenedToReferTag,
    TB_ARTPercTrueCasesTag,
    TB_ARTDiagCaseValuesTag,
    TB_ARTDiagCaseIDsTag,
    TB_ARTDiagCaseMethodsTag,
    TB_ARTDiagCaseSensitivityTag,
    TB_ARTDiagCaseSpecificityTag,
    TB_ARTNumDiagnosedPTBTag,
    TB_ARTNumConfirmedPTBTag,
    TB_ARTPropLinkedTreatTag,
    TB_ARTNumInitTreatmentTag,
    TB_ARTPrvntInActiveEligibleTag,
    TB_ARTPropPrvntPresumpTag,
    TB_ARTPropPrvntTestedTag,
    TB_ARTPrvntMethodsTag,
    TB_ARTPrvntMethodIDsTag,
    TB_ARTPropEligiblePrvntLinkTag,
    TB_ARTScreeningOutputTag,
    TB_ARTDiagnosticOutputTag,
    TB_ARTTargetPopsTag,
    TB_HRGDemographicsTag,
    TB_HRGProportionTag,
    TB_HRGScreeningAlgTag,
    TB_HRGScreeningAlgIDTag,
    TB_HRGScreenSensitivityTag,
    TB_HRGScreenSpecificityTag,
    TB_HRGScreenAlgOptionsTag,
    TB_HRGScreenAlgSelectedTag,
    TB_HRGDiagSensitivityTag,
    TB_HRGDiagSpecificityTag,
    TB_HRGNumScreenedTag,
    TB_HRGNumReferredTag,
    TB_HRGNumScreenedToReferTag,
    TB_HRGPercTrueCasesTag,
    TB_HRGDiagCaseValuesTag,
    TB_HRGDiagCaseIDsTag,
    TB_HRGDiagCaseMethodsTag,
    TB_HRGDiagCaseSensitivityTag,
    TB_HRGDiagCaseSpecificityTag,
    TB_HRGNumDiagnosedPTBTag,
    TB_HRGNumConfirmedPTBTag,
    TB_HRGPropLinkedTreatTag,
    TB_HRGNumInitTreatmentTag,
    TB_HRGPrvntInActiveEligibleTag,
    TB_HRGPropPrvntPresumpTag,
    TB_HRGPropPrvntTestedTag,
    TB_HRGPrvntMethodsTag,
    TB_HRGPrvntMethodIDsTag,
    TB_HRGPropEligiblePrvntLinkTag,
    TB_HRGScreeningOutputTag,
    TB_HRGDiagnosticOutputTag,
    TB_HRGTargetPopsTag,
    TB_PatientScreenSensTag,
    TB_PatientScreenSpecTag,
    TB_DiagSensTag,
    TB_DiagSpecTag,
    TB_PulmonaryAgeTag,
    TB_PrevalenceTag,
    TB_PatientScreenSensitivityTag,
    TB_PatientScreenSpecificityTag,
    TB_PatientScreenAlgOptionsTag,
    TB_PatientScreenAlgSelectedTag,
    TB_NumClinicallyAssessedTag,
    TB_NumReferredTag,
    TB_NumScreenedToReferTag,
    TB_PercTrueCasesTag,
    TB_PatientDiagCaseValuesTag,
    TB_PatientDiagCaseIDsTag,
    TB_PatientDiagCaseMethodsTag,
    TB_PatientDiagCaseSensitivityTag,
    TB_PatientDiagCaseSpecificityTag,
    TB_PatientDiagSensitivityTag,
    TB_PatientDiagSpecificityTag,
    TB_NumDiagnosedPTBTag,
    TB_NumConfirmedPTBTag,
    TB_NumDiagnosedTBTag,
    TB_NumInitTreatmentTag,
    TB_PatientScreeningOutputTag,
    TB_PatientDiagnosticOutputTag,
    TB_PatientTargetPopTag,
    TB_HisNotificationTag,
    TB_TargNotificationTag,
    TB_ResNotificationTag,
    TB_ResNotificationV2Tag,
    TB_ResPrevalenceTag,
    TB_ResPrevalencePctTag,
    TB_ResHIVTBCoInfTag,
    TB_ResIncidenceTag,
    TB_ResMortalityTag,
    TB_CounterfactualScenarioTag,
    TB_CurrentScenarioTag,
    TB_FortIncidenceTag,
    TB_FortNotificationTag,
    TB_FortPrevalenceTag,
    TB_FortMortalityTag,
    TB_NotificationByCaseTypeTag,
    TB_PercNotificationTag,
    TB_Baseline_HRi_allTag,
    TB_Scaleup_HRi_allTag,
    TB_ResistProfileTag,
    TB_ResistPropNonSevereTag,
    TB_ResistPropLungSurgeryTag,
    TB_ResistMeningitisTag,
    TB_ResistPropTreatExhaustTag,
    TB_PatientTestNumPTB,
    TB_PatientTestNumXDR,
    TB_PatientTestNumEPTB,
    TB_CoverageTag,
    TB_ResistNumToBeTestedTag,
    TB_ResistNumTestedTag,
    TB_ResistNumConfirmedTag,
    TB_ResistNumNotConfirmedTag,
    TB_PropPreventionTag,
    TB_BCGImmunizationTag,
    TB_InfantVaccineTag,
    TB_AdolAdultVaccineTag,
    TB_PalliativeCareTag,
    TB_BCGTargetPopTag,
    TB_InfantVaccTargPopTag,
    TB_AdultVaccTargPopTag,
    TB_PalliativeCareTargPopTag,
    TB_PreventionTargetTag,
    TB_RR_DST_PTBTag,
    TB_RR_DST_ETBTag,
    TB_INH_DST_PTBTag,
    TB_INH_DST_ETBTag,
    TB_FQ_DST_PTBTag,
    TB_FQ_DST_ETBTag,
    TB_BDQ_DST_PTBTag,
    TB_BDQ_DST_ETBTag,
    TB_BDQ_FQ_DST_PTBTag,
    TB_BDQ_FQ_DST_ETBTag,
    TB_LZD_RR_DST_PTBTag,
    TB_LZD_RR_DST_ETBTag,
    TB_LZD_FQ_DST_PTBTag,
    TB_LZD_FQ_DST_ETBTag,
    TB_PZA_DSTTag,
    TB_ETO_DSTTag,
    TB_EMC_DSTTag,
    TB_AmikacynRifampicinResistant,
    TB_EthionamideRifampicinResistant,
    TB_PyrazinamideRifampicinResistant,
    TB_Rif_INH_Sen_SevereTag,
    TB_Rif_INH_Sen_NotSevereTag,
    TB_Rif_INH_Sen_MeningitisTag,
    TB_Rif_INH_ResTag,
    TB_RR_FQ_SenTag,
    TB_Rif_FQ_Res_PlusTag,
    TB_BQD_LZD_Tag,
    TB_TxFL_Tag,
    TB_TxSL_Tag,
    TB_TxPreXDR_Tag,
    TB_LungSurgeryTag,
    TB_lPTB_ReferredDxTag,
    TB_HIVnClinicallyAssessedTag,
    TB_HIVAnyReferredDxTag,
    TB_DynProportionsTag,
    TB_DynRatesTag,
    TB_DynPRMTag  ,
    TB_DynUsedFitParamTag,
    TB_DynUsedHIVIncdTag,
    TB_DynRangesTag,
    TB_FortVsnTag,
    
    TB_PatientCustStepMenu ,
    TB_PatientCustStep1    ,
    TB_PatientCustStep2    ,
    TB_PatientCustParallel,

    TB_HHCustStepMenu,
    TB_HHCustStep1,
    TB_HHCustStep2,
    TB_HHCustParallel,

    TB_ARTCustStepMenu,
    TB_ARTCustStep1,
    TB_ARTCustStep2,
    TB_ARTCustParallel,

    TB_HRGCustStepMenu,
    TB_HRGCustStep1,
    TB_HRGCustStep2,
    TB_HRGCustParallel
]


